function clone() {
    var newObj = {};
    for (prop in this) {
        newObj[prop] = this[prop];
    }
    return newObj;
}


// HTML/Font.php > HTML/Font.js
function Font() {
    return {
        family: '',
        name: '',
        charset: '',
        codepage: '',
        toStyle: function() {
            var list = [];
            if (this.name) list.push(this.name);
            if (this.family) list.push(this.family);
            if (list.length == 0) return "";
            return "font-family: " + list.join("") + ";";
        },
        isFont: true, // esta substituindo instanceof
        clone: clone
    }
}

//---------------------------- END FONT -------------------------------

// HTML/HtmlFormatter.php -> HTML/HtmlFormatter.js

function HtmlFormatter(encoding) { // default UTF-8
    encoding = encoding ? encoding : 'UTF-8';
    return {
        output: "",
        encoding: encoding,
        defaultFont: "",
        mapping: [],
        isHtmlFormatter: true,
        divOpen: false,
        Format: function(document) {
            // Keep track of style modifications
            this.previousState = null;
            // and create a stack of states
            this.states = [];
            // Put an initial standard state onto the stack
            this.state = State();
            array_push(this.states, this.state);

            // Keep track of opened html tags
            this.openedTags = {
                'div': false,
                'span': false,
                'p': false
            };
            // Create the first paragraph
            this.OpenTag('p');
            // Begin format
            this.ProcessGroup(document.root);
            // Remove any final unclosed 'p' tag and return result:
            return this.openedTags['p'] ? substr(this.output, 0, -3) : this.output;

        },
        LoadFont: function(fontGroup) {
            fontNumber = 0;
            font = Font();

            // Loop through children of the font group. The font group
            // contains control words with the font number and charset,
            // and a control text with the font name.
            for (var i = 0; i < fontGroup.children.length; i++) {
                var child = fontGroup.children[i];

                // Control word
                if (child.isControlWord) {
                    switch (child.word) {
                        case 'f':
                            fontNumber = child.parameter;
                            break;

                            // Font family names
                        case 'froman':
                            font.family = "serif";
                            break;
                        case 'fswiss':
                            font.family = "sans-serif";
                            break;
                        case 'fmodern':
                            font.family = "monospace";
                            break;
                        case 'fscript':
                            font.family = "cursive";
                            break;
                        case 'fdecor':
                            font.family = "fantasy";
                            break;
						case 'fcharset':
							font.family = ""

                            // case 'fnil': break; // default font
                            // case 'ftech': break; // symbol
                            // case 'fbidi': break; // bidirectional font                      

                        case 'fcharset': // charset
                            font.charset = this.GetEncodingFromCharset(child.parameter);
                            break;
                        case 'cpg': // code page
                            font.codepage = this.GetEncodingFromCodepage(child.parameter);
                            break;
                        case 'fprq': // Font pitch
                            font.fprq = child.parameter;
                            break;
                    }
                }

                // Control text contains the font name, if any:
                if (child.isText) {
                    // Store font name (except ; delimiter at end)
                    font.name = substr(child.text, 0, -1);
                }

            }
            GlobalState.SetFont(fontNumber, font);
        },
        ExtractFontTable: function(fontTblGrp) {
            for (var i = 0; i < fontTblGrp.children.length; i++) {
                var child = fontTblGrp.children[i];
                if (!child.isGroup) continue;
                this.LoadFont(child);
            }
        },
        ExtractColorTable: function(colorTblGrp) {
            // {\colortbl;\red0\green0\blue0;}
            // Index 0 of the RTF color table  is the 'auto' color
            colortbl = [];
            c = count(colorTblGrp);
            color = '';
            for (var i = 1; i < c; i++) { // Iterate through colors
                if (colorTblGrp[i].isControlWord) {
                    // Extract RGB color and convert it to hex string
                    color = sprintf('#%02x%02x%02x', // hex string format
                        colorTblGrp[i].parameter, // red
                        colorTblGrp[i + 1].parameter, // green
                        colorTblGrp[i + 2].parameter); // blue
                    i += 2;
                } else if (colorTblGrp[i].isText) {
                    // This is a delimiter ';' so
                    if (i != 1) { // Store the already extracted color
                        colortbl.push(color);
                    } else { // This is the 'auto' color
                        colortbl.push(0);
                    }
                }
            }
            GlobalState.colortbl = colortbl;
        },
        ExtractImage: function(pictGrp) {
            Image = Image();
            for (var i = 0; i < pictGrp.length; i++) {
                var child = pictGrp[i];
                if (child.isControlWord) {
                    switch (child.word) {
                        // Picture Format
                        case "emfblip":
                            Image.format = 'emf';
                            break;
                        case "pngblip":
                            Image.format = 'png';
                            break;
                        case "jpegblip":
                            Image.format = 'jpeg';
                            break;
                        case "macpict":
                            Image.format = 'pict';
                            break;
                            // case "wmetafile": Image.format = 'bmp'; break;

                            // Picture size and scaling
                        case "picw":
                            Image.width = child.parameter;
                            break;
                        case "pich":
                            Image.height = child.parameter;
                            break;
                        case "picwgoal":
                            Image.goalWidth = child.parameter;
                            break;
                        case "pichgoal":
                            Image.goalHeight = child.parameter;
                            break;
                        case "picscalex":
                            Image.pcScaleX = child.parameter;
                            break;
                        case "picscaley":
                            Image.pcScaleY = child.parameter;
                            break;

                            // Binary or Hexadecimal Data ?
                        case "bin":
                            Image.binarySize = child.parameter;
                            break;
                        default:
                            break;
                    }

                } else if (child.isText) { // store Data
                    Image.ImageData = child.text;
                }
            }
            // output Image
            this.output += Image.PrintImage();
            delete Image;
        },
        ProcessGroup: function(group) {
            // Special group processing:
            switch (group.GetType()) {
                case "fonttbl": // Extract font table
                    this.ExtractFontTable(group);
                    return;
                case "colortbl": // Extract color table
                    this.ExtractColorTable(group.children);
                    return;
                case "stylesheet":
                    // Stylesheet extraction not yet supported
                    return;
                case "info":
                    // Ignore Document information
                    return;
                case "pict":
                    this.ExtractImage(group.children);
                    return;
                case "nonshppict":
                    // Ignore alternative images
                    return;
                case "*": // Process destination
                    this.ProcessDestination(group.children);
                    return;
            }

            // Pictures extraction not yet supported
            //if(substr(group.GetType(), 0, 4) == "pict") return;

            // Push a new state onto the stack:
            this.state = this.state.clone();
            array_push(this.states, this.state);

            for (var i = 0; i < group.children.length; i++) {
                var child = group.children[i];
                this.FormatEntry(child);
            }

            // Pop state from stack
            array_pop(this.states);
            this.state = this.states[sizeof(this.states) - 1];
        },
        ProcessDestination: function(dest) {
            if (!dest[1].isControlWord) return;
            // Check if this is a Word 97 picture
            if (dest[1].word == "shppict") {
                c = count(dest);
                for (i = 2; i < c; i++) {
                    this.FormatEntry(dest[i]);
                }
            }

        },
        FormatEntry: function(entry) {
            if (entry.isGroup) {
                this.ProcessGroup(entry);
            } else if (entry.isControlWord) {
                this.FormatControlWord(entry);
            } else if (entry.isControlSymbol) {
                this.FormatControlSymbol(entry);
            } else if (entry.isText) {
                this.FormatText(entry);
            }
        },
        FormatControlWord: function(word) {
            switch (word.word) {
                case 'plain': // Reset font formatting properties to default.
                case 'pard': // Reset to default paragraph properties.
                    this.state.Reset(this.defaultFont);
                    break;

                    /*
                     * Font formatting properties:
                     */

                case 'b': // bold
                    this.state.bold = word.parameter;
                    break;
                case 'i': // italic
                    this.state.italic = word.parameter;
                    break;
                case 'ul': // underline
                    this.state.underline = word.parameter;
                    break;
                case 'ulnone': // no underline 
                    this.state.underline = false;
                    break;
                case 'strike': // strike-through 
                    this.state.strike = word.parameter;
                    break;
                case 'v': // hidden 
                    this.state.hidden = word.parameter;
                    break;
                case 'fs': // Font size 
                    fontSize = ceil((word.parameter / 24) * 16);
                    GlobalState.addCacheFontSize(fontSize);
                    this.state.fontsize = fontSize;
                    break;
                case 'f': // Font
                    this.state.font = word.parameter;
                    break;
                case 'deff': // Store default font
                    this.defaultFont = word.parameter;
                    break;
                case 'qc':
                    this.state.align = 'text-align: center;';
                    break;
                case 'qr':
                    this.state.align = 'text-align: right;';
                    break;
                    /*
                     * Colors
                     */

                case 'cf':
                case 'chcfpat':
                    this.state.fontcolor = word.parameter;
                    break;
                case 'cb':
                case 'chcbpat':
                    this.state.background = word.parameter;
                    break;
                case 'highlight':
                    this.state.hcolor = word.parameter;
                    break;

                    /*
                     * Special characters
                     */

                case 'lquote':
                    this.Write("&lsquo;");
                    break; // &#145; &#8216;
                case 'rquote':
                    this.Write("&rsquo;");
                    break; // &#146; &#8217;
                case 'ldblquote':
                    this.Write("&ldquo;");
                    break; // &#147; &#8220;
                case 'rdblquote':
                    this.Write("&rdquo;");
                    break; // &#148; &#8221;
                case 'bullet':
                    this.Write("&bull;");
                    break; // &#149; &#8226;
                case 'endash':
                    this.Write("&ndash;");
                    break; // &#150; &#8211;
                case 'emdash':
                    this.Write("&mdash;");
                    break; // &#151; &#8212;
                case 'enspace':
                    this.Write("&ensp;");
                    break; // &#8194;
                case 'emspace':
                    this.Write("&emsp;");
                    break; // &#8195;
                case 'tab':
                    this.Write("&nbsp;");
                    break; // Character value 9
                case 'line':
                    this.output += "<br/>";
                    break; // character value (line feed = &#10;) (carriage return = &#13;)

                    /*
                     * Unicode characters 
                     */

                case 'u':
                    uchar = this.DecodeUnicode(word.parameter);
                    this.Write(uchar);
                    break;

                    /*
                     * Paragraphs
                     */
                case 'par':
                case 'row':
                    // Close previously opened tags
                    this.CloseTags();
                    // Begin a new paragraph
                    this.OpenTag('p');
                    break;

                    /* Code pages */
                case 'ansi':
                case 'mac':
                case 'pc':
                case 'pca':
                    this.RTFencoding = this.GetEncodingFromCodepage(word.word);
                    break;
                case 'ansicpg':
                    if (word.parameter) {
                        this.RTFencoding = this.GetEncodingFromCodepage(word.parameter);
                    }
                    break;
            }

        },
        DecodeUnicode: function(code, srcEnc) {
            srcEnc = !srcEnc ? 'UTF-8' : srcEnc;
            utf8 = '';

            if (srcEnc != 'UTF-8') { // convert character to Unicode
                utf8 = encode(chr(code)); //iconv(srcEnc, 'UTF-8', chr(code));
            }

            if (this.encoding == 'HTML-ENTITIES') {
                return utf8 ? "&#" + this.ord_utf8(utf8) + ";" : "&#" + code + ";";
            } else if (this.encoding == 'UTF-8') {
                return utf8 ? utf8 : he.encode("&#" + code + ";"); //mb_convert_encoding("&#{code};", this.encoding, 'HTML-ENTITIES');
            } else {
                return utf8 ? mb_convert_encoding(utf8, this.encoding, 'UTF-8') :
                    mb_convert_encoding("&#{code};", this.encoding, 'HTML-ENTITIES');
            }
        },
        Write: function(txt) {
            if (
                !this.state.equals(this.previousState) ||
                (this.state.equals(this.previousState) &&
                    !this.openedTags['span'])
            ) {
                if (this.openedTags['span']) {
                    this.CloseTag('span');
                }
                if (this.divOpen == true) {
                    this.CloseTag('div');
                    this.divOpen = false;
                }
                style = this.state.PrintStyle();

                // Keep track of preceding style
                this.previousState = this.state.clone();

                // Create style attribute and open span
                attr = style ? "style=\"" + style + "\"" : "";

                this.OpenTag('span', attr);
                if (strpos(style, "align") !== false) {
                    this.OpenTag('div', attr);
                    this.divOpen = true;
                }
            }

            this.output += txt;

        },
        OpenTag: function(tag, attr) {
            attr = !attr ? "" : attr;
            this.output += attr ? "<" + tag + " " + attr + ">" : "<" + tag + ">";
            this.openedTags[tag] = true;
        },
        CloseTag: function(tag) {
            if (this.openedTags[tag]) {
                // Check for empty html elements
                if (substr(this.output, -strlen("<" + tag + ">")) == "<" + tag + ">") {
                    switch (tag) {
                        case 'p': // Replace empty 'p' element with a line break
                            this.output = substr(this.output, 0, -3) + "<br>";
                            break;
                        default: // Delete empty elements
                            this.output = substr(this.output, 0, -strlen("<" + tag + ">"));
                            break;
                    }
                } else {
                    this.output += "</" + tag + ">";
                }
                this.openedTags[tag] = false;
            }
        },
        CloseTags: function() {
            for (var tag in this.openedTags) {
                this.CloseTag(tag);
            }
        },
        FormatControlSymbol: function(symbol) {
            if (symbol.symbol == '\'') {
                enc = this.GetSourceEncoding();
                uchar = this.DecodeUnicode(symbol.parameter, enc);
                this.Write(uchar);
            } else if (symbol.symbol == '~') {
                this.Write("&nbsp;"); // Non breaking space
            } else if (symbol.symbol == '-') {
                this.Write("&#173;"); // Optional hyphen
            } else if (symbol.symbol == '_') {
                this.Write("&#8209;"); // Non breaking hyphen
            }
        },
        FormatText: function(text) {
            txt = htmlspecialchars(text.text, 'ENT_NOQUOTES', 'UTF-8');
            if (this.encoding == 'HTML-ENTITIES')
                this.Write(txt);
            else
                this.Write(he.encode(txt)); //this.Write(mb_convert_encoding(txt, this.encoding, 'UTF-8'));
        },
        GetSourceEncoding: function() {
            if (isset(this.state.font)) {
                if (isset(GlobalState.fonttbl[this.state.font].codepage)) {
                    return GlobalState.fonttbl[this.state.font].codepage;

                } else if (isset(GlobalState.fonttbl[this.state.font].charset)) {
                    return GlobalState.fonttbl[this.state.font].charset;
                }
            }
            return this.RTFencoding;
        },
        GetEncodingFromCharset: function(fcharset) {
            /* maps windows character sets to iconv encoding names */
            var charset = {
                0: 'CP1252', // ANSI: Western Europe
                1: 'CP1252', //*Default
                2: 'CP1252', //*Symbol
                3: null, // Invalid
                77: 'MAC', //*also [MacRoman]: Macintosh 
                128: 'CP932', //*or [Shift_JIS]?: Japanese
                129: 'CP949', //*also [UHC]: Korean (Hangul)
                130: 'CP1361', //*also [JOHAB]: Korean (Johab)
                134: 'CP936', //*or [GB2312]?: Simplified Chinese
                136: 'CP950', //*or [BIG5]?: Traditional Chinese             
                161: 'CP1253', // Greek
                162: 'CP1254', // Turkish (latin 5)
                163: 'CP1258', // Vietnamese
                177: 'CP1255', // Hebrew
                178: 'CP1256', // Simplified Arabic
                179: 'CP1256', //*Traditional Arabic
                180: 'CP1256', //*Arabic User
                181: 'CP1255', //*Hebrew User
                186: 'CP1257', // Baltic
                204: 'CP1251', // Russian (Cyrillic)
                222: 'CP874', // Thai
                238: 'CP1250', // Eastern European (latin 2)
                254: 'CP437', //*also [IBM437][437]: PC437
                255: 'CP437'
            }; //*OEM still PC437

            if (isset(charset[fcharset]))
                return charset[fcharset];
            else {
                throw new Error("Unknown charset: " + fcharset);
            }
        },
        GetEncodingFromCodepage: function(cpg) {
            codePage = {
                'ansi': 'CP1252',
                'mac': 'MAC',
                'pc': 'CP437',
                'pca': 'CP850',
                437: 'CP437', // United States IBM          
                708: 'ASMO-708', // also [ISO-8859-6][ARABIC] Arabic
                /*  Not supported by iconv
                709,: '' // Arabic (ASMO 449+, BCON V4) 
                710,: '' // Arabic (transparent Arabic) 
                711,: '' // Arabic (Nafitha Enhanced) 
                720,: '' // Arabic (transparent ASMO) 
                */
                819: 'CP819', // Windows 3.1 (US and Western Europe) 
                850: 'CP850', // IBM multilingual 
                852: 'CP852', // Eastern European 
                860: 'CP860', // Portuguese 
                862: 'CP862', // Hebrew 
                863: 'CP863', // French Canadian 
                864: 'CP864', // Arabic 
                865: 'CP865', // Norwegian 
                866: 'CP866', // Soviet Union 
                874: 'CP874', // Thai 
                932: 'CP932', // Japanese 
                936: 'CP936', // Simplified Chinese 
                949: 'CP949', // Korean 
                950: 'CP950', // Traditional Chinese 
                1250: 'CP1250', // Windows 3.1 (Eastern European) 
                1251: 'CP1251', // Windows 3.1 (Cyrillic) 
                1252: 'CP1252', // Western European 
                1253: 'CP1253', // Greek 
                1254: 'CP1254', // Turkish 
                1255: 'CP1255', // Hebrew 
                1256: 'CP1256', // Arabic 
                1257: 'CP1257', // Baltic 
                1258: 'CP1258', // Vietnamese 
                1361: 'CP1361'
            }; // Johab

            if (isset(codePage[cpg]))
                return codePage[cpg];
            else {
                // Debug Error
                throw new Error("Unknown codepage: " + cpg);
            }
        },
        ord_utf8: function(chr) {
            ord0 = ord(chr);
            if (ord0 >= 0 && ord0 <= 127)
                return ord0;

            ord1 = ord(chr[1]);
            if (ord0 >= 192 && ord0 <= 223)
                return (ord0 - 192) * 64 + (ord1 - 128);

            ord2 = ord(chr[2]);
            if (ord0 >= 224 && ord0 <= 239)
                return (ord0 - 224) * 4096 + (ord1 - 128) * 64 + (ord2 - 128);

            ord3 = ord(chr[3]);
            if (ord0 >= 240 && ord0 <= 247)
                return (ord0 - 240) * 262144 + (ord1 - 128) * 4096 + (ord2 - 128) * 64 + (ord3 - 128);

            ord4 = ord(chr[4]);
            if (ord0 >= 248 && ord0 <= 251)
                return (ord0 - 248) * 16777216 + (ord1 - 128) * 262144 + (ord2 - 128) * 4096 + (ord3 - 128) * 64 + (ord4 - 128);

            if (ord0 >= 252 && ord0 <= 253)
                return (ord0 - 252) * 1073741824 + (ord1 - 128) * 16777216 + (ord2 - 128) * 262144 + (ord3 - 128) * 4096 + (ord4 - 128) * 64 + (ord(chr[5]) - 128);

            throw new Error("Invalid Unicode character: " + chr);
            //trigger_error("Invalid Unicode character: {chr}");

        }
    };
}

//---------------------------- END HTML_FORMATTER ---------------------
// HTML/Image.php -> HTML/Image.js 
function Image() {
    return {
        format: 'bmp',
        width: 0, // in xExt if wmetafile otherwise in px
        height: 0, // in yExt if wmetafile otherwise in px
        goalWidth: 0, // in twips
        goalHeight: 0, // in twips
        pcScaleX: 100, // 100%
        pcScaleY: 100, // 100%
        binarySize: null, // Number of bytes of the binary data
        ImageData: null, // Binary or Hexadecimal Data
        isImage: true,
        PrintImage: function() {
            // <img src="data:image/{FORMAT};base64,{#BDATA}" />
            output = "<img src=\"data:image/" + this.format + ";base64,";

            if (isset(this.binarySize)) { // process binary data
                return;
            } else { // process hexadecimal data
                output += base64_encode(pack('H*', this.ImageData));
            }

            output += "\" />";
            return output;
        },
        Reset: function() {
            this.format = 'bmp';
            this.width = 0; // in xExt if wmetafile otherwise in px
            this.height = 0; // in yExt if wmetafile otherwise in px
            this.goalWidth = 0; // in twips
            this.goalHeight = 0; // in twips
            this.pcScaleX = 100; // 100%
            this.pcScaleY = 100; // 100%
            this.binarySize = null; // Number of bytes of the binary data
            this.ImageData = null; // Binary or Hexadecimal Data
        },
        clone: clone
    }
}
//----------------------------- END IMAGE -----------------------------

// HTML/State.php -> HTML/State.js 

function State() {
    return {
        fonttbl: [],
        colortbl: [],
        cacheFontSize: [],
        highlight: {
            1: 'Black',
            2: 'Blue',
            3: 'Cyan',
            4: 'Green',
            5: 'Magenta',
            6: 'Red',
            7: 'Yellow',
            8: 'Unused',
            9: 'DarkBlue',
            10: 'DarkCyan',
            11: 'DarkGreen',
            12: 'DarkMagenta',
            13: 'DarkRed',
            14: 'DarkYellow',
            15: 'DarkGray',
            16: 'LightGray'
        },
        bold: false,
        italic: false,
        underline: false,
        strike: false,
        hidden: false,
        fontsize: 0,
        fontcolor: null,
        background: null,
        hcolor: null,
        font: null,
        align: 'text-align: left;',
        isState: true,
        clone: clone,
        SetFont: function(index, font) {
            GlobalState.fonttbl[index] = font;
        },
        addCacheFontSize: function(fontSize) {
            this.cacheFontSize.push(fontSize);
        },
        getLastFontSize: function() {
            lastIndex = this.cacheFontSize.length - 1;
            if (lastIndex == -1) return false;
            return this.cacheFontSize[this.cacheFontSize.length - 1];
        },
        Reset: function(defaultFont) {
            this.bold = false;
            this.italic = false;
            this.underline = false;
            this.strike = false;
            this.hidden = false;
            this.fontsize = 0;
            this.fontcolor = null;
            this.background = null;
            this.hcolor = null;
            this.font = isset(defaultFont) ? defaultFont : null;
            this.align = 'text-align: left;';
        },
        PrintStyle: function() {
            style = "";

            if (this.bold) style += "font-weight:bold;";
            if (this.italic) style += "font-style:italic;";
            if (this.underline) style += "text-decoration:underline;";
            if (this.align && this.align !== 'text-align: left;') style += this.align;
            // state->underline is a toggle switch variable so no need for
            // a dedicated state->end_underline variable
            // if(this.state->end_underline) {$span += "text-decoration:none;";}
            if (this.strike) style += "text-decoration:line-through;";
            if (this.hidden) style += "display:none;";
            if (isset(this.font)) {
                font = GlobalState.fonttbl[this.font];
                style += font.toStyle();
            }
            lastFontCache = this.getLastFontSize();
            if (this.fontsize == 0 && lastFontCache !== false) this.fontsize = lastFontCache;
            if (this.fontsize != 0) {
                style += "font-size:" + parseInt(this.fontsize) * 2.5 + "px;"; //gabriel verificar verify fix
            }

            // Font color:
            if (isset(this.fontcolor)) {
                // Check if color is set. in particular when it's the 'auto' color
                if (array_key_exists(this.fontcolor, this.colortbl) && this.colortbl[this.fontcolor])
                    style += "color:" + this.colortbl[this.fontcolor] + ";";
            }
            // Background color:
            if (isset(this.background)) {
                // Check if color is set. in particular when it's the 'auto' color
                if (this.colortbl[this.background])
                    style += "background-color:" + this.colortbl[this.background] + ";";

                // Highlight color:
            } else if (isset(this.hcolor)) {
                if (isset(this.highlight[this.hcolor]))
                    style += "background-color:" + this.highlight[this.hcolor] + ";";
            }

            return style;
        },
        equals: function(state) {
            if (!state || !state.isState) return false;

            if (this.bold != state.bold) return false;
            if (this.italic != state.italic) return false;
            if (this.underline != state.underline) return false;
            if (this.strike != state.strike) return false;
            if (this.hidden != state.hidden) return false;
            if (this.fontsize != state.fontsize) return false;

            // Compare colors
            if (this.fontcolor != state.fontcolor) return false;
            if (this.background != state.background) return false;
            if (this.hcolor != state.hcolor) return false;

            // Compare fonts
            if (this.font != state.font) return false;

            return true;
        }
    };
}

// ----------------------------- END STATE ----------------------------

// ControlSymbol.php -> ControlSymbol.js

function ControlSymbol() {
    return {
        symbol: "",
        parameter: 0,
        isControlSymbol: true,
        toString: function(level) {
            return str_repeat("  ", level) + "SYMBOL " + this.symbol + " (" + this.parameter + ")\n";
        }
    };
}

// ---------------------------- END CONTROL_SYMBOL --------------------

// ControlWord.php -> ControlWord.js

function ControlWord() {
    return {
        word: "",
        parameter: "",
        isControlWord: true,
        toString: function(level) {
            return str_repeat("  ", level) + "WORD " + this.word + " (" + this.parameter + ")\n";
        }
    }
}

//------------------------------ END CONTROL_WORD ---------------------

// Document.php -> Document.js

function Document(rtf) {
    doc = {
        rtf: "",
        _char: "",
        pos: "",
        len: "",
        root: null,
        group: "",
        GetChar: function() {
            this._char = null;
            if (this.pos < strlen(this.rtf)) {
                this._char = this.rtf[this.pos++];
            } else {
                err = "Parse error: Tried to read past end of input; RTF is probably truncated." + this.rtf;
                //trigger_error(err);
                throw new Error(err);
            }
        },
        is_letter: function() {
            if (ord(this._char) >= 65 && ord(this._char) <= 90) return true;
            if (ord(this._char) >= 97 && ord(this._char) <= 122) return true;
            return false;
        },
        is_digit: function() {
            return (ord(this._char) >= 48 && ord(this._char) <= 57);
        },
        is_endofline: function() {
            if (this._char == "\r" || this._char == "\n") {
                // Checks for a Windows/Acron type EOL
                if (this.rtf[this.pos] == "\n" || this.rtf[this.pos] == "\r") {
                    this.GetChar();
                }
                return true;
            }
            return false;
        },
        is_space_delimiter: function() {
            return (this._char == " " || this.is_endofline());
        },
        ParseStartGroup: function() {
            group = Group();
            // Is there a current group? Then make the new group its child:
            if (this.group != null) {
                group.parent = this.group;
                array_push(this.group.children, group);
                array_push(this.uc, end(this.uc));
            }
            // If there is no parent group, then set this group
            // as the root group.
            else {
                this.root = group;
                // Create uc stack and insert the first default value
                this.uc = [1];
            }

            // Set the new group as the current group:
            this.group = group;
        },
        ParseEndGroup: function() {
            this.group = this.group.parent;
            // Retrieve last uc value from stack
            array_pop(this.uc);
        },
        ParseControlWord: function() {
            // Read letters until a non-letter is reached.
            word = "";
            this.GetChar();
            while (this.is_letter()) {
                word += this._char;
                this.GetChar();
            }

            // Read parameter (if any) consisting of digits.
            // Parameter may be negative, i.e., starting with a '-'
            parameter = null;
            negative = false;
            if (this._char == '-') {
                this.GetChar();
                negative = true;
            }
            while (this.is_digit()) {
                if (parameter == null) parameter = 0;
                parameter = parseInt(parameter) * 10 + parseInt(this._char); //WARNING
                this.GetChar();
            }
            // If no parameter present, assume control word's default (usually 1)
            // If no default then assign 0 to the parameter
            if (parameter === null) parameter = 1;

            // Convert parameter to a negative number when applicable
            if (negative) parameter = -parameter;

            // Update uc value
            if (word == "uc") {
                array_pop(this.uc);
                this.uc.push(parameter);
            }

            // Skip space delimiter
            if (!this.is_space_delimiter()) this.pos--;

            // If this is \u, then the parameter will be followed 
            // by {this.uc} characters.
            if (word == "u") {
                // Convert parameter to unsigned decimal unicode
                if (negative) parameter = 65536 + parameter;

                // Will ignore replacement characters uc times
                var uc = end(this.uc);
                while (uc > 0) {
                    this.GetChar();
                    // If the replacement character is encoded as
                    // hexadecimal value \'hh then jump over it
                    if (this._char == '\\' && this.rtf[this.pos] == '\'') {
                        this.pos = this.pos + 3;

                        // Break if it's an RTF scope delimiter
                    } else if (this._char == '{' || this._char == '{') {
                        break;
                    }

                    // - To include an RTF delimiter in skippable data, it must be
                    //   represented using the appropriate control symbol (that is,
                    //   escaped with a backslash,) as in plain text.
                    // 
                    // - Any RTF control word or symbol is considered a single character
                    //   for the purposes of counting skippable characters. For this reason
                    //   it's more appropriate to create a skip flag and let the Parse()
                    //   function take care of the skippable characters.
                    uc--;
                }
            }

            // Add new RTF word as a child to the current group.
            rtfword = ControlWord();
            rtfword.word = word;
            rtfword.parameter = parameter;
            array_push(this.group.children, rtfword);
        },
        ParseControlSymbol: function() {
            // Read symbol (one character only).
            this.GetChar();
            symbol = this._char;

            // Exceptional case: 
            // Treat EOL symbols as \par control word
            if (this.is_endofline()) {
                rtfword = new ControlWord();
                rtfword.word = 'par';
                rtfword.parameter = 0;
                array_push(this.group.children, rtfword);
                return;
            }

            // Symbols ordinarily have no parameter. However, 
            // if this is \' (a single quote), then it is 
            // followed by a 2-digit hex-code:
            parameter = 0;
            if (symbol == '\'') {
                this.GetChar();
                parameter = this._char;
                this.GetChar();
                parameter = hexdec(parameter + this._char);
            }

            // Add new control symbol as a child to the current group:
            rtfsymbol = ControlSymbol();
            rtfsymbol.symbol = symbol;
            rtfsymbol.parameter = parameter;
            array_push(this.group.children, rtfsymbol);
        },
        ParseControl: function() {
            // Beginning of an RTF control word or control symbol.
            // Look ahead by one character to see if it starts with
            // a letter (control world) or another symbol (control symbol):
            this.GetChar();
            this.pos--; // (go back after look-ahead)
            if (this.is_letter()) {
                this.ParseControlWord();
            } else {
                this.ParseControlSymbol();
            }
        },
        ParseText: function() {
            // Parse plain text up to backslash or brace,
            // unless escaped.
            text = "";
            terminate = false;
            do {
                // Ignore EOL characters
                if (this._char == "\r" || this._char == "\n") {
                    this.GetChar();
                    continue;
                }
                // Is this an escape?
                if (this._char == '\\') {
                    // Perform lookahead to see if this
                    // is really an escape sequence.
                    this.GetChar();
                    switch (this._char) {
                        case '\\':
                            break;
                        case '{':
                            break;
                        case '}':
                            break;
                        default:
                            // Not an escape. Roll back.
                            this.pos = this.pos - 2;
                            terminate = true;
                            break;
                    }
                } else if (this._char == '{' || this._char == '}') {
                    this.pos--;
                    terminate = true;
                }

                if (!terminate) {
                    // Save plain text
                    text += this._char;
                    this.GetChar();
                }
            }
            while (!terminate && this.pos < this.len);

            // Create new Text element:
            text = Text(text);

            // If there is no current group, then this is not a valid RTF file.
            // Throw an exception.
            if (this.group == null) {
                err = "Parse error: RTF text outside of group.";
                //trigger_error(err);
                throw new Error(err);
            }

            // Add text as a child to the current group:
            array_push(this.group.children, text);
        },
        Parse: function(rtf) {
            this.rtf = rtf;
            this.pos = 0;
            this.len = strlen(this.rtf);
            this.group = null;
            this.root = null;

            while (this.pos < this.len) {
                // Read next character:
                this.GetChar();

                // Ignore \r and \n
                if (this._char == "\n" || this._char == "\r") continue;

                // What type of character is this?
                switch (this._char) {
                    case '{':
                        this.ParseStartGroup();
                        break;
                    case '}':
                        this.ParseEndGroup();
                        break;
                    case '\\':
                        this.ParseControl();
                        break;
                    default:
                        this.ParseText();
                        break;
                }
            }
        },
        __toString: function() {
            if (!this.root) return "No root group";
            return this.root.toString();
        }
    };
    doc.Parse(rtf);
    return doc;
}

// ------------------------------ END DOCUMENT ------------------------

// Element.php -> Element.js

function Element() {
    return {};
}

// ------------------------------- END ELEMENT ------------------------

// Group.php -> Group.js

function Group() {
    var el = Element();
    el.parent = null;
    el.children = [];
    el.isGroup = true;
    el.GetType = function() {
        if (sizeof(this.children) == 0) return null;

        child = this.children[0];
        if (child.isControlWord) {
            return child.word;

        } else if (child.isControlSymbol) {
            return (child.symbol == '*') ? '*' : null;
        }

        return null;
    };
    el.IsDestination = function() {
        if (sizeof(this.children) == 0) return null;
        child = this.children[0];
        if (!child.isControlSymbol) return null;
        return child.symbol == '*';
    };
    el.toString = function(level) {
        level = !level ? 0 : level;
        str = str_repeat("  ", level) + "{\n";
        for (i = 0; i < this.children.length; i++) {
            var child = this.children[i];
            str += child.toString(level + 1);
        }
        return str + str_repeat("  ", level) + "}\n";
    };
    return el;
}
// ------------------------------- END GROUP ------------------------

// Text.php -> Text.js

function Text(text) {
    var el = Element();
    el.text = text;
    el.isText = true;
    el.toString = function() {
        return str_repeat("  ", level) + "TEXT " + this.text + "\n";
    };
    return el;
}

// ------------------------------- END TEXT ------------------------
//----------------------------- NATIVE FUNCTIONS ----------------------

function count(arr) {
    return arr.length;
}

function pack(format) {
    //  discuss at: https://locutus.io/php/pack/
    // original by: Tim de Koning (https://www.kingsquare.nl)
    //    parts by: Jonas Raoni Soares Silva (https://www.jsfromhell.com)
    // bugfixed by: Tim de Koning (https://www.kingsquare.nl)
    //      note 1: Float encoding by: Jonas Raoni Soares Silva
    //      note 1: Home: https://www.kingsquare.nl/blog/12-12-2009/13507444
    //      note 1: Feedback: phpjs-pack@kingsquare.nl
    //      note 1: "machine dependent byte order and size" aren't
    //      note 1: applicable for JavaScript; pack works as on a 32bit,
    //      note 1: little endian machine.
    //   example 1: pack('nvc*', 0x1234, 0x5678, 65, 66)
    //   returns 1: '\u00124xVAB'
    //   example 2: pack('H4', '2345')
    //   returns 2: '#E'
    //   example 3: pack('H*', 'D5')
    //   returns 3: 'Õ'
    //   example 4: pack('d', -100.876)
    //   returns 4: "\u0000\u0000\u0000\u0000\u00008YÀ"
    //        test: skip-1
    var formatPointer = 0;
    var argumentPointer = 1;
    var result = '';
    var argument = '';
    var i = 0;
    var r = [];
    var instruction, quantifier, word, precisionBits, exponentBits, extraNullCount;
    // vars used by float encoding
    var bias;
    var minExp;
    var maxExp;
    var minUnnormExp;
    var status;
    var exp;
    var len;
    var bin;
    var signal;
    var n;
    var intPart;
    var floatPart;
    var lastBit;
    var rounded;
    var j;
    var k;
    var tmpResult;
    while (formatPointer < format.length) {
        instruction = format.charAt(formatPointer)
        quantifier = ''
        formatPointer++
        while ((formatPointer < format.length) && (format.charAt(formatPointer)
                .match(/[\d*]/) !== null)) {
            quantifier += format.charAt(formatPointer)
            formatPointer++
        }
        if (quantifier === '') {
            quantifier = '1'
        }
        // Now pack variables: 'quantifier' times 'instruction'
        switch (instruction) {
            case 'a':
            case 'A':
                // NUL-padded string
                // SPACE-padded string
                if (typeof arguments[argumentPointer] === 'undefined') {
                    throw new Error('Warning:  pack() Type ' + instruction + ': not enough arguments')
                } else {
                    argument = String(arguments[argumentPointer])
                }
                if (quantifier === '*') {
                    quantifier = argument.length
                }
                for (i = 0; i < quantifier; i++) {
                    if (typeof argument[i] === 'undefined') {
                        if (instruction === 'a') {
                            result += String.fromCharCode(0)
                        } else {
                            result += ' '
                        }
                    } else {
                        result += argument[i]
                    }
                }
                argumentPointer++
                break
            case 'h':
            case 'H':
                // Hex string, low nibble first
                // Hex string, high nibble first
                if (typeof arguments[argumentPointer] === 'undefined') {
                    throw new Error('Warning: pack() Type ' + instruction + ': not enough arguments')
                } else {
                    argument = arguments[argumentPointer]
                }
                if (quantifier === '*') {
                    quantifier = argument.length
                }
                if (quantifier > argument.length) {
                    const msg = 'Warning: pack() Type ' + instruction + ': not enough characters in string'
                    throw new Error(msg)
                }
                for (i = 0; i < quantifier; i += 2) {
                    // Always get per 2 bytes...
                    word = argument[i]
                    if (((i + 1) >= quantifier) || typeof argument[i + 1] === 'undefined') {
                        word += '0'
                    } else {
                        word += argument[i + 1]
                    }
                    // The fastest way to reverse?
                    if (instruction === 'h') {
                        word = word[1] + word[0]
                    }
                    result += String.fromCharCode(parseInt(word, 16))
                }
                argumentPointer++
                break
            case 'c':
            case 'C':
                // signed char
                // unsigned char
                // c and C is the same in pack
                if (quantifier === '*') {
                    quantifier = arguments.length - argumentPointer
                }
                if (quantifier > (arguments.length - argumentPointer)) {
                    throw new Error('Warning:  pack() Type ' + instruction + ': too few arguments')
                }
                for (i = 0; i < quantifier; i++) {
                    result += String.fromCharCode(arguments[argumentPointer])
                    argumentPointer++
                }
                break
            case 's':
            case 'S':
            case 'v':
                // signed short (always 16 bit, machine byte order)
                // unsigned short (always 16 bit, machine byte order)
                // s and S is the same in pack
                if (quantifier === '*') {
                    quantifier = arguments.length - argumentPointer
                }
                if (quantifier > (arguments.length - argumentPointer)) {
                    throw new Error('Warning:  pack() Type ' + instruction + ': too few arguments')
                }
                for (i = 0; i < quantifier; i++) {
                    result += String.fromCharCode(arguments[argumentPointer] & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] >> 8 & 0xFF)
                    argumentPointer++
                }
                break
            case 'n':
                // unsigned short (always 16 bit, big endian byte order)
                if (quantifier === '*') {
                    quantifier = arguments.length - argumentPointer
                }
                if (quantifier > (arguments.length - argumentPointer)) {
                    throw new Error('Warning: pack() Type ' + instruction + ': too few arguments')
                }
                for (i = 0; i < quantifier; i++) {
                    result += String.fromCharCode(arguments[argumentPointer] >> 8 & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] & 0xFF)
                    argumentPointer++
                }
                break
            case 'i':
            case 'I':
            case 'l':
            case 'L':
            case 'V':
                // signed integer (machine dependent size and byte order)
                // unsigned integer (machine dependent size and byte order)
                // signed long (always 32 bit, machine byte order)
                // unsigned long (always 32 bit, machine byte order)
                // unsigned long (always 32 bit, little endian byte order)
                if (quantifier === '*') {
                    quantifier = arguments.length - argumentPointer
                }
                if (quantifier > (arguments.length - argumentPointer)) {
                    throw new Error('Warning:  pack() Type ' + instruction + ': too few arguments')
                }
                for (i = 0; i < quantifier; i++) {
                    result += String.fromCharCode(arguments[argumentPointer] & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] >> 8 & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] >> 16 & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] >> 24 & 0xFF)
                    argumentPointer++
                }
                break
            case 'N':
                // unsigned long (always 32 bit, big endian byte order)
                if (quantifier === '*') {
                    quantifier = arguments.length - argumentPointer
                }
                if (quantifier > (arguments.length - argumentPointer)) {
                    throw new Error('Warning:  pack() Type ' + instruction + ': too few arguments')
                }
                for (i = 0; i < quantifier; i++) {
                    result += String.fromCharCode(arguments[argumentPointer] >> 24 & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] >> 16 & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] >> 8 & 0xFF)
                    result += String.fromCharCode(arguments[argumentPointer] & 0xFF)
                    argumentPointer++
                }
                break
            case 'f':
            case 'd':
                // float (machine dependent size and representation)
                // double (machine dependent size and representation)
                // version based on IEEE754
                precisionBits = 23
                exponentBits = 8
                if (instruction === 'd') {
                    precisionBits = 52
                    exponentBits = 11
                }
                if (quantifier === '*') {
                    quantifier = arguments.length - argumentPointer
                }
                if (quantifier > (arguments.length - argumentPointer)) {
                    throw new Error('Warning:  pack() Type ' + instruction + ': too few arguments')
                }
                for (i = 0; i < quantifier; i++) {
                    argument = arguments[argumentPointer]
                    bias = Math.pow(2, exponentBits - 1) - 1
                    minExp = -bias + 1
                    maxExp = bias
                    minUnnormExp = minExp - precisionBits
                    status = isNaN(n = parseFloat(argument)) || n === -Infinity || n === +Infinity ? n : 0
                    exp = 0
                    len = 2 * bias + 1 + precisionBits + 3
                    bin = new Array(len)
                    signal = (n = status !== 0 ? 0 : n) < 0
                    n = Math.abs(n)
                    intPart = Math.floor(n)
                    floatPart = n - intPart
                    for (k = len; k;) {
                        bin[--k] = 0
                    }
                    for (k = bias + 2; intPart && k;) {
                        bin[--k] = intPart % 2
                        intPart = Math.floor(intPart / 2)
                    }
                    for (k = bias + 1; floatPart > 0 && k; --floatPart) {
                        (bin[++k] = ((floatPart *= 2) >= 1) - 0)
                    }
                    for (k = -1; ++k < len && !bin[k];) {}
                    // @todo: Make this more readable:
                    const key = (lastBit = precisionBits - 1 +
                        (k =
                            (exp = bias + 1 - k) >= minExp &&
                            exp <= maxExp ? k + 1 : bias + 1 - (exp = minExp - 1))) + 1
                    if (bin[key]) {
                        if (!(rounded = bin[lastBit])) {
                            for (j = lastBit + 2; !rounded && j < len; rounded = bin[j++]) {}
                        }
                        for (j = lastBit + 1; rounded && --j >= 0;
                            (bin[j] = !bin[j] - 0) && (rounded = 0)) {}
                    }
                    for (k = k - 2 < 0 ? -1 : k - 3; ++k < len && !bin[k];) {}
                    if ((exp = bias + 1 - k) >= minExp && exp <= maxExp) {
                        ++k
                    } else {
                        if (exp < minExp) {
                            if (exp !== bias + 1 - len && exp < minUnnormExp) {
                                // "encodeFloat::float underflow"
                            }
                            k = bias + 1 - (exp = minExp - 1)
                        }
                    }
                    if (intPart || status !== 0) {
                        exp = maxExp + 1
                        k = bias + 2
                        if (status === -Infinity) {
                            signal = 1
                        } else if (isNaN(status)) {
                            bin[k] = 1
                        }
                    }
                    n = Math.abs(exp + bias)
                    tmpResult = ''
                    for (j = exponentBits + 1; --j;) {
                        tmpResult = (n % 2) + tmpResult
                        n = n >>= 1
                    }
                    n = 0
                    j = 0
                    k = (tmpResult = (signal ? '1' : '0') + tmpResult + (bin
                        .slice(k, k + precisionBits)
                        .join(''))).length
                    r = []
                    for (; k;) {
                        n += (1 << j) * tmpResult.charAt(--k)
                        if (j === 7) {
                            r[r.length] = String.fromCharCode(n)
                            n = 0
                        }
                        j = (j + 1) % 8
                    }
                    r[r.length] = n ? String.fromCharCode(n) : ''
                    result += r.join('')
                    argumentPointer++
                }
                break
            case 'x':
                // NUL byte
                if (quantifier === '*') {
                    throw new Error('Warning: pack(): Type x: \'*\' ignored')
                }
                for (i = 0; i < quantifier; i++) {
                    result += String.fromCharCode(0)
                }
                break
            case 'X':
                // Back up one byte
                if (quantifier === '*') {
                    throw new Error('Warning: pack(): Type X: \'*\' ignored')
                }
                for (i = 0; i < quantifier; i++) {
                    if (result.length === 0) {
                        throw new Error('Warning: pack(): Type X:' + ' outside of string')
                    } else {
                        result = result.substring(0, result.length - 1)
                    }
                }
                break
            case '@':
                // NUL-fill to absolute position
                if (quantifier === '*') {
                    throw new Error('Warning: pack(): Type X: \'*\' ignored')
                }
                if (quantifier > result.length) {
                    extraNullCount = quantifier - result.length
                    for (i = 0; i < extraNullCount; i++) {
                        result += String.fromCharCode(0)
                    }
                }
                if (quantifier < result.length) {
                    result = result.substring(0, quantifier)
                }
                break
            default:
                throw new Error('Warning: pack() Type ' + instruction + ': unknown format code')
        }
    }
    if (argumentPointer < arguments.length) {
        const msg2 = 'Warning: pack(): ' + (arguments.length - argumentPointer) + ' arguments unused'
        throw new Error(msg2)
    }
    return result
}

function sprintf() {
    const regex = /%%|%(?:(\d+)\$)?((?:[-+#0 ]|'[\s\S])*)(\d+)?(?:\.(\d*))?([\s\S])/g;
    const args = arguments;
    var i = 0;
    const format = args[i++];
    const _pad = function(str, len, chr, leftJustify) {
        if (!chr) {
            chr = ' ';
        }
        const padding = (str.length >= len) ? '' : new Array(1 + len - str.length >>> 0).join(chr);
        return leftJustify ? str + padding : padding + str;
    }
    const justify = function(value, prefix, leftJustify, minWidth, padChar) {
        const diff = minWidth - value.length;
        if (diff > 0) {
            // when padding with zeros
            // on the left side
            // keep sign (+ or -) in front
            if (!leftJustify && padChar === '0') {
                value = [
                    value.slice(0, prefix.length),
                    _pad('', diff, '0', true),
                    value.slice(prefix.length)
                ].join('');
            } else {
                value = _pad(value, minWidth, padChar, leftJustify);
            }
        }
        return value;
    }
    const _formatBaseX = function(value, base, leftJustify, minWidth, precision, padChar) {
        // Note: casts negative numbers to positive ones
        const number = value >>> 0;
        value = _pad(number.toString(base), precision || 0, '0', false);
        return justify(value, '', leftJustify, minWidth, padChar);
    }
    // _formatString()
    const _formatString = function(value, leftJustify, minWidth, precision, customPadChar) {
        if (precision !== null && precision !== undefined) {
            value = value.slice(0, precision);
        }
        return justify(value, '', leftJustify, minWidth, customPadChar);
    }
    // doFormat()
    const doFormat = function(substring, argIndex, modifiers, minWidth, precision, specifier) {
        var number, prefix, method, textTransform, value;
        if (substring === '%%') {
            return '%';
        }
        // parse modifiers
        var padChar = ' '; // pad with spaces by default
        var leftJustify = false;
        var positiveNumberPrefix = '';
        var j, l;
        for (j = 0, l = modifiers.length; j < l; j++) {
            switch (modifiers.charAt(j)) {
                case ' ':
                case '0':
                    padChar = modifiers.charAt(j);
                    break;
                case '+':
                    positiveNumberPrefix = '+';
                    break;
                case '-':
                    leftJustify = true;
                    break;
                case "'":
                    if (j + 1 < l) {
                        padChar = modifiers.charAt(j + 1);
                        j++;
                    }
                    break;
            }
        }
        if (!minWidth) {
            minWidth = 0;
        } else {
            minWidth = +minWidth;
        }
        if (!isFinite(minWidth)) {
            throw new Error('Width must be finite');
        }
        if (!precision) {
            precision = (specifier === 'd') ? 0 : 'fFeE'.indexOf(specifier) > -1 ? 6 : undefined;
        } else {
            precision = +precision;
        }
        if (argIndex && +argIndex === 0) {
            throw new Error('Argument number must be greater than zero');
        }
        if (argIndex && +argIndex >= args.length) {
            throw new Error('Too few arguments');
        }
        value = argIndex ? args[+argIndex] : args[i++];
        switch (specifier) {
            case '%':
                return '%';
            case 's':
                return _formatString(value + '', leftJustify, minWidth, precision, padChar);
            case 'c':
                return _formatString(String.fromCharCode(+value), leftJustify, minWidth, precision, padChar);
            case 'b':
                return _formatBaseX(value, 2, leftJustify, minWidth, precision, padChar);
            case 'o':
                return _formatBaseX(value, 8, leftJustify, minWidth, precision, padChar);
            case 'x':
                return _formatBaseX(value, 16, leftJustify, minWidth, precision, padChar);
            case 'X':
                return _formatBaseX(value, 16, leftJustify, minWidth, precision, padChar)
                    .toUpperCase();
            case 'u':
                return _formatBaseX(value, 10, leftJustify, minWidth, precision, padChar);
            case 'i':
            case 'd':
                number = +value || 0;
                // Plain Math.round doesn't just truncate
                number = Math.round(number - number % 1);
                prefix = number < 0 ? '-' : positiveNumberPrefix;
                value = prefix + _pad(String(Math.abs(number)), precision, '0', false);
                if (leftJustify && padChar === '0') {
                    // can't right-pad 0s on integers
                    padChar = ' ';
                }
                return justify(value, prefix, leftJustify, minWidth, padChar);
            case 'e':
            case 'E':
            case 'f': // @todo: Should handle locales (as per setlocale)
            case 'F':
            case 'g':
            case 'G':
                number = +value;
                prefix = number < 0 ? '-' : positiveNumberPrefix;
                method = ['toExponential', 'toFixed', 'toPrecision']['efg'.indexOf(specifier.toLowerCase())];
                textTransform = ['toString', 'toUpperCase']['eEfFgG'.indexOf(specifier) % 2];
                value = prefix + Math.abs(number)[method](precision);
                return justify(value, prefix, leftJustify, minWidth, padChar)[textTransform]();
            default:
                // unknown specifier, consume that char and return empty
                return '';
        }
    }
    try {
        return format.replace(regex, doFormat);
    } catch (err) {
        return false
    }
}

function isset() {
    //  discuss at: https://locutus.io/php/isset/
    // original by: Kevin van Zonneveld (https://kvz.io)
    // improved by: FremyCompany
    // improved by: Onno Marsman (https://twitter.com/onnomarsman)
    // improved by: Rafał Kukawski (https://blog.kukawski.pl)
    //   example 1: isset( undefined, true)
    //   returns 1: false
    //   example 2: isset( 'Kevin van Zonneveld' )
    //   returns 2: true
    const a = arguments;
    const l = a.length;
    var i = 0;
    var undef;
    if (l === 0) {
        throw new Error('Empty isset');
    }
    while (i !== l) {
        if (a[i] === undef || a[i] === null) {
            return false;
        }
        i++;
    }
    return true;
}

function btoa(string) {

    var b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
    // Regular expression to check formal correctness of base64 encoded strings
    var b64re = /^(?:[A-Za-z\d+\/]{4})*?(?:[A-Za-z\d+\/]{2}(?:==)?|[A-Za-z\d+\/]{3}=?)?$/;
    string = String(string);
    var bitmap, a, b, c,
        result = "",
        i = 0,
        rest = string.length % 3; // To determine the final padding

    for (; i < string.length;) {
        if ((a = string.charCodeAt(i++)) > 255 ||
            (b = string.charCodeAt(i++)) > 255 ||
            (c = string.charCodeAt(i++)) > 255)
            throw new TypeError("Failed to execute 'btoa' on 'Window': The string to be encoded contains characters outside of the Latin1 range.");

        bitmap = (a << 16) | (b << 8) | c;
        result += b64.charAt(bitmap >> 18 & 63) + b64.charAt(bitmap >> 12 & 63) +
            b64.charAt(bitmap >> 6 & 63) + b64.charAt(bitmap & 63);
    }

    // If there's need of padding, replace the last 'A's with equal signs
    return rest ? result.slice(0, rest - 3) + "===".substring(rest) : result;
};

function base64_encode(stringToEncode) { // eslint-disable-line camelcase
    //  discuss at: https://locutus.io/php/base64_encode/
    // original by: Tyler Akins (https://rumkin.com)
    // improved by: Bayron Guevara
    // improved by: Thunder.m
    // improved by: Kevin van Zonneveld (https://kvz.io)
    // improved by: Kevin van Zonneveld (https://kvz.io)
    // improved by: Rafał Kukawski (https://blog.kukawski.pl)
    // bugfixed by: Pellentesque Malesuada
    // improved by: Indigo744
    //   example 1: base64_encode('Kevin van Zonneveld')
    //   returns 1: 'S2V2aW4gdmFuIFpvbm5ldmVsZA=='
    //   example 2: base64_encode('a')
    //   returns 2: 'YQ=='
    //   example 3: base64_encode('✓ à la mode')
    //   returns 3: '4pyTIMOgIGxhIG1vZGU='
    // encodeUTF8string()
    // Internal function to encode properly UTF8 string
    // Adapted from Solution #1 at https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding
    const encodeUTF8string = function(str) {
        // first we use encodeURIComponent to get percent-encoded UTF-8,
        // then we convert the percent encodings into raw bytes which
        // can be fed into the base64 encoding algorithm.
        return encodeURIComponent(str).replace(/%([0-9A-F]{2})/g,
            function toSolidBytes(match, p1) {
                return String.fromCharCode('0x' + p1);
            });
    };
    if (typeof window !== 'undefined') {
        if (typeof window.btoa !== 'undefined') {
            return window.btoa(encodeUTF8string(stringToEncode));
        }
    } else if (typeof btoa == "function") {
        return btoa(encodeUTF8string(stringToEncode));
    } else {
        return new Buffer(stringToEncode).toString('base64')
    }
    const b64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
    var o1;
    var o2;
    var o3;
    var h1;
    var h2;
    var h3;
    var h4;
    var bits;
    var i = 0;
    var ac = 0;
    var enc = '';
    const tmpArr = []
    if (!stringToEncode) {
        return stringToEncode
    }
    stringToEncode = encodeUTF8string(stringToEncode)
    do {
        // pack three octets into four hexets
        o1 = stringToEncode.charCodeAt(i++)
        o2 = stringToEncode.charCodeAt(i++)
        o3 = stringToEncode.charCodeAt(i++)
        bits = o1 << 16 | o2 << 8 | o3
        h1 = bits >> 18 & 0x3f
        h2 = bits >> 12 & 0x3f
        h3 = bits >> 6 & 0x3f
        h4 = bits & 0x3f
        // use hexets to index into b64, and append result to encoded string
        tmpArr[ac++] = b64.charAt(h1) + b64.charAt(h2) + b64.charAt(h3) + b64.charAt(h4)
    } while (i < stringToEncode.length)
    enc = tmpArr.join('')
    const r = stringToEncode.length % 3
    return (r ? enc.slice(0, r - 3) : enc) + '==='.slice(r || 3)
}

function array_key_exists(key, search) {
    // eslint-disable-line camelcase
    //  discuss at: https://locutus.io/php/array_key_exists/
    // original by: Kevin van Zonneveld (https://kvz.io)
    // improved by: Felix Geisendoerfer (https://www.debuggable.com/felix)
    //   example 1: array_key_exists('kevin', {'kevin': 'van Zonneveld'})
    //   returns 1: true
    if (!search || (search.constructor !== Array && search.constructor !== Object)) {
        return false
    }
    return key in search
}

function str_repeat(input, multiplier) { // eslint-disable-line camelcase
    //  discuss at: https://locutus.io/php/str_repeat/
    // original by: Kevin van Zonneveld (https://kvz.io)
    // improved by: Jonas Raoni Soares Silva (https://www.jsfromhell.com)
    // improved by: Ian Carter (https://euona.com/)
    //   example 1: str_repeat('-=', 10)
    //   returns 1: '-=-=-=-=-=-=-=-=-=-='
    var y = '';
    while (true) {
        if (multiplier & 1) {
            y += input
        }
        multiplier >>= 1
        if (multiplier) {
            input += input
        } else {
            break
        }
    }
    return y
}

function ord(str) {
    return str.charCodeAt();
}

function array_push(arr, item) {
    arr.push(item);
}

function end(arr) {
    lastIndex = arr.length - 1;
    if (lastIndex == -1) return false;
    return arr[lastIndex];
}

function array_pop(arr) {
    return arr.pop();
}

function hexdec(hexString) {
    //  discuss at: https://locutus.io/php/hexdec/
    // original by: Philippe Baumann
    //   example 1: hexdec('that')
    //   returns 1: 10
    //   example 2: hexdec('a0')
    //   returns 2: 160
    hexString = (hexString + '').replace(/[^a-f0-9]/gi, '');
    return parseInt(hexString, 16);
}

function strlen(str) {
    return str.length;
}

function sizeof(arr) {
    return arr.length;
}

function substr(input, start, len) {
    //  discuss at: https://locutus.io/php/substr/
    // original by: Martijn Wieringa
    // bugfixed by: T.Wild
    // improved by: Onno Marsman (https://twitter.com/onnomarsman)
    // improved by: Brett Zamir (https://brett-zamir.me)
    //  revised by: Theriault (https://github.com/Theriault)
    //  revised by: Rafał Kukawski
    //      note 1: Handles rare Unicode characters if 'unicode.semantics' ini (PHP6) is set to 'on'
    //   example 1: substr('abcdef', 0, -1)
    //   returns 1: 'abcde'
    //   example 2: substr(2, 0, -6)
    //   returns 2: false
    //   example 3: ini_set('unicode.semantics', 'on')
    //   example 3: substr('a\uD801\uDC00', 0, -1)
    //   returns 3: 'a'
    //   example 4: ini_set('unicode.semantics', 'on')
    //   example 4: substr('a\uD801\uDC00', 0, 2)
    //   returns 4: 'a\uD801\uDC00'
    //   example 5: ini_set('unicode.semantics', 'on')
    //   example 5: substr('a\uD801\uDC00', -1, 1)
    //   returns 5: '\uD801\uDC00'
    //   example 6: ini_set('unicode.semantics', 'on')
    //   example 6: substr('a\uD801\uDC00z\uD801\uDC00', -3, 2)
    //   returns 6: '\uD801\uDC00z'
    //   example 7: ini_set('unicode.semantics', 'on')
    //   example 7: substr('a\uD801\uDC00z\uD801\uDC00', -3, -1)
    //   returns 7: '\uD801\uDC00z'
    //        test: skip-3 skip-4 skip-5 skip-6 skip-7
    input = input.toString();

    const multibyte = false;
    if (multibyte) {
        input = input.match(/[\uD800-\uDBFF][\uDC00-\uDFFF]|[\s\S]/g) || [];
    }
    const inputLength = input.length;
    var end = inputLength;
    if (start < 0) {
        start += end;
    }
    if (typeof len !== 'undefined') {
        if (len < 0) {
            end = len + end;
        } else {
            end = len + start;
        }
    }
    if (start > inputLength || start < 0 || start > end) {
        return false;
    }
    if (multibyte) {
        return input.slice(start, end).join('');
    }
    return input.slice(start, end);
}

utf8 = {

    stringFromCharCode: String.fromCharCode,

    // Taken from https://mths.be/punycode
    ucs2decode: function(string) {
        var output = [];
        var counter = 0;
        var length = string.length;
        var value;
        var extra;
        while (counter < length) {
            value = string.charCodeAt(counter++);
            if (value >= 0xD800 && value <= 0xDBFF && counter < length) {
                // high surrogate, and there is a next character
                extra = string.charCodeAt(counter++);
                if ((extra & 0xFC00) == 0xDC00) { // low surrogate
                    output.push(((value & 0x3FF) << 10) + (extra & 0x3FF) + 0x10000);
                } else {
                    // unmatched surrogate; only append this code unit, in case the next
                    // code unit is the high surrogate of a surrogate pair
                    output.push(value);
                    counter--;
                }
            } else {
                output.push(value);
            }
        }
        return output;
    },

    // Taken from https://mths.be/punycode
    ucs2encode: function(array) {
        var length = array.length;
        var index = -1;
        var value;
        var output = '';
        while (++index < length) {
            value = array[index];
            if (value > 0xFFFF) {
                value -= 0x10000;
                output += this.stringFromCharCode(value >>> 10 & 0x3FF | 0xD800);
                value = 0xDC00 | value & 0x3FF;
            }
            output += this.stringFromCharCode(value);
        }
        return output;
    },

    checkScalarValue: function(codePoint) {
        if (codePoint >= 0xD800 && codePoint <= 0xDFFF) {
            throw Error(
                'Lone surrogate U+' + codePoint.toString(16).toUpperCase() +
                ' is not a scalar value'
            );
        }
    },
    /*--------------------------------------------------------------------------*/

    createByte: function(codePoint, shift) {
        return this.stringFromCharCode(((codePoint >> shift) & 0x3F) | 0x80);
    },

    encodeCodePoint: function(codePoint) {
        if ((codePoint & 0xFFFFFF80) == 0) { // 1-byte sequence
            return this.stringFromCharCode(codePoint);
        }
        var symbol = '';
        if ((codePoint & 0xFFFFF800) == 0) { // 2-byte sequence
            symbol = this.stringFromCharCode(((codePoint >> 6) & 0x1F) | 0xC0);
        } else if ((codePoint & 0xFFFF0000) == 0) { // 3-byte sequence
            this.checkScalarValue(codePoint);
            symbol = this.stringFromCharCode(((codePoint >> 12) & 0x0F) | 0xE0);
            symbol += this.createByte(codePoint, 6);
        } else if ((codePoint & 0xFFE00000) == 0) { // 4-byte sequence
            symbol = this.stringFromCharCode(((codePoint >> 18) & 0x07) | 0xF0);
            symbol += this.createByte(codePoint, 12);
            symbol += this.createByte(codePoint, 6);
        }
        symbol += this.stringFromCharCode((codePoint & 0x3F) | 0x80);
        return symbol;
    },

    utf8encode: function(string) {
        var codePoints = this.ucs2decode(string);
        var length = codePoints.length;
        var index = -1;
        var codePoint;
        var byteString = '';
        while (++index < length) {
            codePoint = codePoints[index];
            byteString += this.encodeCodePoint(codePoint);
        }
        return byteString;
    },

    /*--------------------------------------------------------------------------*/

    readContinuationByte: function() {
        if (this.byteIndex >= this.byteCount) {
            throw Error('Invalid byte index');
        }

        var continuationByte = this.byteArray[this.byteIndex] & 0xFF;
        this.byteIndex++;

        if ((continuationByte & 0xC0) == 0x80) {
            return continuationByte & 0x3F;
        }

        // If we end up here, it’s not a continuation byte
        throw Error('Invalid continuation byte');
    },

    decodeSymbol: function() {
        var byte1;
        var byte2;
        var byte3;
        var byte4;
        var codePoint;

        if (this.byteIndex > this.byteCount) {
            throw Error('Invalid byte index');
        }

        if (this.byteIndex == this.byteCount) {
            return false;
        }

        // Read first byte
        byte1 = this.byteArray[this.byteIndex] & 0xFF;
        this.byteIndex++;

        // 1-byte sequence (no continuation bytes)
        if ((byte1 & 0x80) == 0) {
            return byte1;
        }

        // 2-byte sequence
        if ((byte1 & 0xE0) == 0xC0) {
            byte2 = this.readContinuationByte();
            codePoint = ((byte1 & 0x1F) << 6) | byte2;
            if (codePoint >= 0x80) {
                return codePoint;
            } else {
                throw Error('Invalid continuation byte');
            }
        }

        // 3-byte sequence (may include unpaired surrogates)
        if ((byte1 & 0xF0) == 0xE0) {
            byte2 = this.readContinuationByte();
            byte3 = this.readContinuationByte();
            codePoint = ((byte1 & 0x0F) << 12) | (byte2 << 6) | byte3;
            if (codePoint >= 0x0800) {
                this.checkScalarValue(codePoint);
                return codePoint;
            } else {
                throw Error('Invalid continuation byte');
            }
        }

        // 4-byte sequence
        if ((byte1 & 0xF8) == 0xF0) {
            byte2 = this.readContinuationByte();
            byte3 = this.readContinuationByte();
            byte4 = this.readContinuationByte();
            codePoint = ((byte1 & 0x07) << 0x12) | (byte2 << 0x0C) |
                (byte3 << 0x06) | byte4;
            if (codePoint >= 0x010000 && codePoint <= 0x10FFFF) {
                return codePoint;
            }
        }

        throw Error('Invalid UTF-8 detected');
    },

    byteArray: null,
    byteCount: null,
    byteIndex: null,
    utf8decode: function(byteString) {
        this.byteArray = this.ucs2decode(byteString);
        this.byteCount = this.byteArray.length;
        this.byteIndex = 0;
        var codePoints = [];
        var tmp;
        while ((tmp = this.decodeSymbol()) !== false) {
            codePoints.push(tmp);
        }
        return this.ucs2encode(codePoints);
    },

    /*--------------------------------------------------------------------------*/

    version: '3.0.0'

};
utf8.decode = utf8.utf8decode;
utf8.encode = utf8.utf8encode;

function chr(codePt) {
    //  discuss at: https://locutus.io/php/chr/
    // original by: Kevin van Zonneveld (https://kvz.io)
    // improved by: Brett Zamir (https://brett-zamir.me)
    //   example 1: chr(75) === 'K'
    //   example 1: chr(65536) === '\uD800\uDC00'
    //   returns 1: true
    //   returns 1: true
    if (codePt > 0xFFFF) { // Create a four-byte string (length 2) since this code point is high
        //   enough for the UTF-16 encoding (JavaScript internal use), to
        //   require representation with two surrogates (reserved non-characters
        //   used for building other characters; the first is "high" and the next "low")
        codePt -= 0x10000
        return String.fromCharCode(0xD800 + (codePt >> 10), 0xDC00 + (codePt & 0x3FF))
    }
    return String.fromCharCode(codePt)
}

function htmlspecialchars(string, quoteStyle, charset, doubleEncode) {
    //       discuss at: https://locutus.io/php/htmlspecialchars/
    //      original by: Mirek Slugen
    //      improved by: Kevin van Zonneveld (https://kvz.io)
    //      bugfixed by: Nathan
    //      bugfixed by: Arno
    //      bugfixed by: Brett Zamir (https://brett-zamir.me)
    //      bugfixed by: Brett Zamir (https://brett-zamir.me)
    //       revised by: Kevin van Zonneveld (https://kvz.io)
    //         input by: Ratheous
    //         input by: Mailfaker (https://www.weedem.fr/)
    //         input by: felix
    // reimplemented by: Brett Zamir (https://brett-zamir.me)
    //           note 1: charset argument not supported
    //        example 1: htmlspecialchars("<a href='test'>Test</a>", 'ENT_QUOTES')
    //        returns 1: '&lt;a href=&#039;test&#039;&gt;Test&lt;/a&gt;'
    //        example 2: htmlspecialchars("ab\"c'd", ['ENT_NOQUOTES', 'ENT_QUOTES'])
    //        returns 2: 'ab"c&#039;d'
    //        example 3: htmlspecialchars('my "&entity;" is still here', null, null, false)
    //        returns 3: 'my &quot;&entity;&quot; is still here'
    var optTemp = 0;
    var i = 0;
    var noquotes = false;
    if (typeof quoteStyle === 'undefined' || quoteStyle === null) {
        quoteStyle = 2;
    }
    string = string || '';
    string = string.toString();
    if (doubleEncode !== false) {
        // Put this first to avoid double-encoding
        //string = string.replace(/&/g, '&amp;');
    }
    //string = string.replace(/</g, '&lt;').replace(/>/g, '&gt;');
    const OPTS = {
        ENT_NOQUOTES: 0,
        ENT_HTML_QUOTE_SINGLE: 1,
        ENT_HTML_QUOTE_DOUBLE: 2,
        ENT_COMPAT: 2,
        ENT_QUOTES: 3,
        ENT_IGNORE: 4
    }
    if (quoteStyle === 0) {
        noquotes = true;
    }
    if (typeof quoteStyle !== 'number') {
        // Allow for a single string or an array of string flags
        quoteStyle = [].concat(quoteStyle);
        for (i = 0; i < quoteStyle.length; i++) {
            // Resolve string input to bitwise e.g. 'ENT_IGNORE' becomes 4
            if (OPTS[quoteStyle[i]] === 0) {
                noquotes = true;
            } else if (OPTS[quoteStyle[i]]) {
                optTemp = optTemp | OPTS[quoteStyle[i]];
            }
        }
        quoteStyle = optTemp;
    }
    if (quoteStyle & OPTS.ENT_HTML_QUOTE_SINGLE) {
        string = string.replace(/'/g, '&#039;');
    }
    if (!noquotes) {
        string = string.replace(/"/g, '&quot;');
    }
    return string;
}

function ceil(value) {
    //  discuss at: https://locutus.io/php/ceil/
    // original by: Onno Marsman (https://twitter.com/onnomarsman)
    //   example 1: ceil(8723321.4)
    //   returns 1: 8723322
    return Math.ceil(value);
}

function strpos(haystack, needle, offset) {
    //  discuss at: https://locutus.io/php/strpos/
    // original by: Kevin van Zonneveld (https://kvz.io)
    // improved by: Onno Marsman (https://twitter.com/onnomarsman)
    // improved by: Brett Zamir (https://brett-zamir.me)
    // bugfixed by: Daniel Esteban
    //   example 1: strpos('Kevin van Zonneveld', 'e', 5)
    //   returns 1: 14
    var i = (haystack + '').indexOf(needle, (offset || 0));
    return i === -1 ? false : i;
}

var htmlEntities = {
    // All astral symbols.
    regexAstralSymbols: /[\uD800-\uDBFF][\uDC00-\uDFFF]/g,
    // All ASCII symbols (not just printable ASCII) except those listed in the
    // first column of the overrides table.
    // https://html.spec.whatwg.org/multipage/syntax.html#table-charref-overrides
    regexAsciiWhitelist: /[\x01-\x7F]/g,
    // All BMP symbols that are not ASCII newlines, printable ASCII symbols, or
    // code points listed in the first column of the overrides table on
    // https://html.spec.whatwg.org/multipage/syntax.html#table-charref-overrides.
    regexBmpWhitelist: /[\x01-\t\x0B\f\x0E-\x1F\x7F\x81\x8D\x8F\x90\x9D\xA0-\uFFFF]/g,

    regexEncodeNonAscii: /<\u20D2|=\u20E5|>\u20D2|\u205F\u200A|\u219D\u0338|\u2202\u0338|\u2220\u20D2|\u2229\uFE00|\u222A\uFE00|\u223C\u20D2|\u223D\u0331|\u223E\u0333|\u2242\u0338|\u224B\u0338|\u224D\u20D2|\u224E\u0338|\u224F\u0338|\u2250\u0338|\u2261\u20E5|\u2264\u20D2|\u2265\u20D2|\u2266\u0338|\u2267\u0338|\u2268\uFE00|\u2269\uFE00|\u226A\u0338|\u226A\u20D2|\u226B\u0338|\u226B\u20D2|\u227F\u0338|\u2282\u20D2|\u2283\u20D2|\u228A\uFE00|\u228B\uFE00|\u228F\u0338|\u2290\u0338|\u2293\uFE00|\u2294\uFE00|\u22B4\u20D2|\u22B5\u20D2|\u22D8\u0338|\u22D9\u0338|\u22DA\uFE00|\u22DB\uFE00|\u22F5\u0338|\u22F9\u0338|\u2933\u0338|\u29CF\u0338|\u29D0\u0338|\u2A6D\u0338|\u2A70\u0338|\u2A7D\u0338|\u2A7E\u0338|\u2AA1\u0338|\u2AA2\u0338|\u2AAC\uFE00|\u2AAD\uFE00|\u2AAF\u0338|\u2AB0\u0338|\u2AC5\u0338|\u2AC6\u0338|\u2ACB\uFE00|\u2ACC\uFE00|\u2AFD\u20E5|[\xA0-\u0113\u0116-\u0122\u0124-\u012B\u012E-\u014D\u0150-\u017E\u0192\u01B5\u01F5\u0237\u02C6\u02C7\u02D8-\u02DD\u0311\u0391-\u03A1\u03A3-\u03A9\u03B1-\u03C9\u03D1\u03D2\u03D5\u03D6\u03DC\u03DD\u03F0\u03F1\u03F5\u03F6\u0401-\u040C\u040E-\u044F\u0451-\u045C\u045E\u045F\u2002-\u2005\u2007-\u2010\u2013-\u2016\u2018-\u201A\u201C-\u201E\u2020-\u2022\u2025\u2026\u2030-\u2035\u2039\u203A\u203E\u2041\u2043\u2044\u204F\u2057\u205F-\u2063\u20AC\u20DB\u20DC\u2102\u2105\u210A-\u2113\u2115-\u211E\u2122\u2124\u2127-\u2129\u212C\u212D\u212F-\u2131\u2133-\u2138\u2145-\u2148\u2153-\u215E\u2190-\u219B\u219D-\u21A7\u21A9-\u21AE\u21B0-\u21B3\u21B5-\u21B7\u21BA-\u21DB\u21DD\u21E4\u21E5\u21F5\u21FD-\u2205\u2207-\u2209\u220B\u220C\u220F-\u2214\u2216-\u2218\u221A\u221D-\u2238\u223A-\u2257\u2259\u225A\u225C\u225F-\u2262\u2264-\u228B\u228D-\u229B\u229D-\u22A5\u22A7-\u22B0\u22B2-\u22BB\u22BD-\u22DB\u22DE-\u22E3\u22E6-\u22F7\u22F9-\u22FE\u2305\u2306\u2308-\u2310\u2312\u2313\u2315\u2316\u231C-\u231F\u2322\u2323\u232D\u232E\u2336\u233D\u233F\u237C\u23B0\u23B1\u23B4-\u23B6\u23DC-\u23DF\u23E2\u23E7\u2423\u24C8\u2500\u2502\u250C\u2510\u2514\u2518\u251C\u2524\u252C\u2534\u253C\u2550-\u256C\u2580\u2584\u2588\u2591-\u2593\u25A1\u25AA\u25AB\u25AD\u25AE\u25B1\u25B3-\u25B5\u25B8\u25B9\u25BD-\u25BF\u25C2\u25C3\u25CA\u25CB\u25EC\u25EF\u25F8-\u25FC\u2605\u2606\u260E\u2640\u2642\u2660\u2663\u2665\u2666\u266A\u266D-\u266F\u2713\u2717\u2720\u2736\u2758\u2772\u2773\u27C8\u27C9\u27E6-\u27ED\u27F5-\u27FA\u27FC\u27FF\u2902-\u2905\u290C-\u2913\u2916\u2919-\u2920\u2923-\u292A\u2933\u2935-\u2939\u293C\u293D\u2945\u2948-\u294B\u294E-\u2976\u2978\u2979\u297B-\u297F\u2985\u2986\u298B-\u2996\u299A\u299C\u299D\u29A4-\u29B7\u29B9\u29BB\u29BC\u29BE-\u29C5\u29C9\u29CD-\u29D0\u29DC-\u29DE\u29E3-\u29E5\u29EB\u29F4\u29F6\u2A00-\u2A02\u2A04\u2A06\u2A0C\u2A0D\u2A10-\u2A17\u2A22-\u2A27\u2A29\u2A2A\u2A2D-\u2A31\u2A33-\u2A3C\u2A3F\u2A40\u2A42-\u2A4D\u2A50\u2A53-\u2A58\u2A5A-\u2A5D\u2A5F\u2A66\u2A6A\u2A6D-\u2A75\u2A77-\u2A9A\u2A9D-\u2AA2\u2AA4-\u2AB0\u2AB3-\u2AC8\u2ACB\u2ACC\u2ACF-\u2ADB\u2AE4\u2AE6-\u2AE9\u2AEB-\u2AF3\u2AFD\uFB00-\uFB04]|\uD835[\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDCCF\uDD04\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDD6B]/g,
    encodeMap: {
        '\xAD': 'shy',
        '\u200C': 'zwnj',
        '\u200D': 'zwj',
        '\u200E': 'lrm',
        '\u2063': 'ic',
        '\u2062': 'it',
        '\u2061': 'af',
        '\u200F': 'rlm',
        '\u200B': 'ZeroWidthSpace',
        '\u2060': 'NoBreak',
        '\u0311': 'DownBreve',
        '\u20DB': 'tdot',
        '\u20DC': 'DotDot',
        '\t': 'Tab',
        '\n': 'NewLine',
        '\u2008': 'puncsp',
        '\u205F': 'MediumSpace',
        '\u2009': 'thinsp',
        '\u200A': 'hairsp',
        '\u2004': 'emsp13',
        '\u2002': 'ensp',
        '\u2005': 'emsp14',
        '\u2003': 'emsp',
        '\u2007': 'numsp',
        '\xA0': 'nbsp',
        '\u205F\u200A': 'ThickSpace',
        '\u203E': 'oline',
        '_': 'lowbar',
        '\u2010': 'dash',
        '\u2013': 'ndash',
        '\u2014': 'mdash',
        '\u2015': 'horbar',
        ',': 'comma',
        ';': 'semi',
        '\u204F': 'bsemi',
        ':': 'colon',
        '\u2A74': 'Colone',
        '!': 'excl',
        '\xA1': 'iexcl',
        '?': 'quest',
        '\xBF': 'iquest',
        '.': 'period',
        '\u2025': 'nldr',
        '\u2026': 'mldr',
        '\xB7': 'middot',
        '\'': 'apos',
        '\u2018': 'lsquo',
        '\u2019': 'rsquo',
        '\u201A': 'sbquo',
        '\u2039': 'lsaquo',
        '\u203A': 'rsaquo',
        '"': 'quot',
        '\u201C': 'ldquo',
        '\u201D': 'rdquo',
        '\u201E': 'bdquo',
        '\xAB': 'laquo',
        '\xBB': 'raquo',
        '(': 'lpar',
        ')': 'rpar',
        '[': 'lsqb',
        ']': 'rsqb',
        '{': 'lcub',
        '}': 'rcub',
        '\u2308': 'lceil',
        '\u2309': 'rceil',
        '\u230A': 'lfloor',
        '\u230B': 'rfloor',
        '\u2985': 'lopar',
        '\u2986': 'ropar',
        '\u298B': 'lbrke',
        '\u298C': 'rbrke',
        '\u298D': 'lbrkslu',
        '\u298E': 'rbrksld',
        '\u298F': 'lbrksld',
        '\u2990': 'rbrkslu',
        '\u2991': 'langd',
        '\u2992': 'rangd',
        '\u2993': 'lparlt',
        '\u2994': 'rpargt',
        '\u2995': 'gtlPar',
        '\u2996': 'ltrPar',
        '\u27E6': 'lobrk',
        '\u27E7': 'robrk',
        '\u27E8': 'lang',
        '\u27E9': 'rang',
        '\u27EA': 'Lang',
        '\u27EB': 'Rang',
        '\u27EC': 'loang',
        '\u27ED': 'roang',
        '\u2772': 'lbbrk',
        '\u2773': 'rbbrk',
        '\u2016': 'Vert',
        '\xA7': 'sect',
        '\xB6': 'para',
        '@': 'commat',
        '*': 'ast',
        '/': 'sol',
        'undefined': null,
        '&': 'amp',
        '#': 'num',
        '%': 'percnt',
        '\u2030': 'permil',
        '\u2031': 'pertenk',
        '\u2020': 'dagger',
        '\u2021': 'Dagger',
        '\u2022': 'bull',
        '\u2043': 'hybull',
        '\u2032': 'prime',
        '\u2033': 'Prime',
        '\u2034': 'tprime',
        '\u2057': 'qprime',
        '\u2035': 'bprime',
        '\u2041': 'caret',
        '`': 'grave',
        '\xB4': 'acute',
        '\u02DC': 'tilde',
        '^': 'Hat',
        '\xAF': 'macr',
        '\u02D8': 'breve',
        '\u02D9': 'dot',
        '\xA8': 'die',
        '\u02DA': 'ring',
        '\u02DD': 'dblac',
        '\xB8': 'cedil',
        '\u02DB': 'ogon',
        '\u02C6': 'circ',
        '\u02C7': 'caron',
        '\xB0': 'deg',
        '\xA9': 'copy',
        '\xAE': 'reg',
        '\u2117': 'copysr',
        '\u2118': 'wp',
        '\u211E': 'rx',
        '\u2127': 'mho',
        '\u2129': 'iiota',
        '\u2190': 'larr',
        '\u219A': 'nlarr',
        '\u2192': 'rarr',
        '\u219B': 'nrarr',
        '\u2191': 'uarr',
        '\u2193': 'darr',
        '\u2194': 'harr',
        '\u21AE': 'nharr',
        '\u2195': 'varr',
        '\u2196': 'nwarr',
        '\u2197': 'nearr',
        '\u2198': 'searr',
        '\u2199': 'swarr',
        '\u219D': 'rarrw',
        '\u219D\u0338': 'nrarrw',
        '\u219E': 'Larr',
        '\u219F': 'Uarr',
        '\u21A0': 'Rarr',
        '\u21A1': 'Darr',
        '\u21A2': 'larrtl',
        '\u21A3': 'rarrtl',
        '\u21A4': 'mapstoleft',
        '\u21A5': 'mapstoup',
        '\u21A6': 'map',
        '\u21A7': 'mapstodown',
        '\u21A9': 'larrhk',
        '\u21AA': 'rarrhk',
        '\u21AB': 'larrlp',
        '\u21AC': 'rarrlp',
        '\u21AD': 'harrw',
        '\u21B0': 'lsh',
        '\u21B1': 'rsh',
        '\u21B2': 'ldsh',
        '\u21B3': 'rdsh',
        '\u21B5': 'crarr',
        '\u21B6': 'cularr',
        '\u21B7': 'curarr',
        '\u21BA': 'olarr',
        '\u21BB': 'orarr',
        '\u21BC': 'lharu',
        '\u21BD': 'lhard',
        '\u21BE': 'uharr',
        '\u21BF': 'uharl',
        '\u21C0': 'rharu',
        '\u21C1': 'rhard',
        '\u21C2': 'dharr',
        '\u21C3': 'dharl',
        '\u21C4': 'rlarr',
        '\u21C5': 'udarr',
        '\u21C6': 'lrarr',
        '\u21C7': 'llarr',
        '\u21C8': 'uuarr',
        '\u21C9': 'rrarr',
        '\u21CA': 'ddarr',
        '\u21CB': 'lrhar',
        '\u21CC': 'rlhar',
        '\u21D0': 'lArr',
        '\u21CD': 'nlArr',
        '\u21D1': 'uArr',
        '\u21D2': 'rArr',
        '\u21CF': 'nrArr',
        '\u21D3': 'dArr',
        '\u21D4': 'iff',
        '\u21CE': 'nhArr',
        '\u21D5': 'vArr',
        '\u21D6': 'nwArr',
        '\u21D7': 'neArr',
        '\u21D8': 'seArr',
        '\u21D9': 'swArr',
        '\u21DA': 'lAarr',
        '\u21DB': 'rAarr',
        '\u21DD': 'zigrarr',
        '\u21E4': 'larrb',
        '\u21E5': 'rarrb',
        '\u21F5': 'duarr',
        '\u21FD': 'loarr',
        '\u21FE': 'roarr',
        '\u21FF': 'hoarr',
        '\u2200': 'forall',
        '\u2201': 'comp',
        '\u2202': 'part',
        '\u2202\u0338': 'npart',
        '\u2203': 'exist',
        '\u2204': 'nexist',
        '\u2205': 'empty',
        '\u2207': 'Del',
        '\u2208': 'in',
        '\u2209': 'notin',
        '\u220B': 'ni',
        '\u220C': 'notni',
        '\u03F6': 'bepsi',
        '\u220F': 'prod',
        '\u2210': 'coprod',
        '\u2211': 'sum',
        '+': 'plus',
        '\xB1': 'pm',
        '\xF7': 'div',
        '\xD7': 'times',
        '<': 'lt',
        '\u226E': 'nlt',
        '<\u20D2': 'nvlt',
        '=': 'equals',
        '\u2260': 'ne',
        '=\u20E5': 'bne',
        '\u2A75': 'Equal',
        '>': 'gt',
        '\u226F': 'ngt',
        '>\u20D2': 'nvgt',
        '\xAC': 'not',
        '|': 'vert',
        '\xA6': 'brvbar',
        '\u2212': 'minus',
        '\u2213': 'mp',
        '\u2214': 'plusdo',
        '\u2044': 'frasl',
        '\u2216': 'setmn',
        '\u2217': 'lowast',
        '\u2218': 'compfn',
        '\u221A': 'Sqrt',
        '\u221D': 'prop',
        '\u221E': 'infin',
        '\u221F': 'angrt',
        '\u2220': 'ang',
        '\u2220\u20D2': 'nang',
        '\u2221': 'angmsd',
        '\u2222': 'angsph',
        '\u2223': 'mid',
        '\u2224': 'nmid',
        '\u2225': 'par',
        '\u2226': 'npar',
        '\u2227': 'and',
        '\u2228': 'or',
        '\u2229': 'cap',
        '\u2229\uFE00': 'caps',
        '\u222A': 'cup',
        '\u222A\uFE00': 'cups',
        '\u222B': 'int',
        '\u222C': 'Int',
        '\u222D': 'tint',
        '\u2A0C': 'qint',
        '\u222E': 'oint',
        '\u222F': 'Conint',
        '\u2230': 'Cconint',
        '\u2231': 'cwint',
        '\u2232': 'cwconint',
        '\u2233': 'awconint',
        '\u2234': 'there4',
        '\u2235': 'becaus',
        '\u2236': 'ratio',
        '\u2237': 'Colon',
        '\u2238': 'minusd',
        '\u223A': 'mDDot',
        '\u223B': 'homtht',
        '\u223C': 'sim',
        '\u2241': 'nsim',
        '\u223C\u20D2': 'nvsim',
        '\u223D': 'bsim',
        '\u223D\u0331': 'race',
        '\u223E': 'ac',
        '\u223E\u0333': 'acE',
        '\u223F': 'acd',
        '\u2240': 'wr',
        '\u2242': 'esim',
        '\u2242\u0338': 'nesim',
        '\u2243': 'sime',
        '\u2244': 'nsime',
        '\u2245': 'cong',
        '\u2247': 'ncong',
        '\u2246': 'simne',
        '\u2248': 'ap',
        '\u2249': 'nap',
        '\u224A': 'ape',
        '\u224B': 'apid',
        '\u224B\u0338': 'napid',
        '\u224C': 'bcong',
        '\u224D': 'CupCap',
        '\u226D': 'NotCupCap',
        '\u224D\u20D2': 'nvap',
        '\u224E': 'bump',
        '\u224E\u0338': 'nbump',
        '\u224F': 'bumpe',
        '\u224F\u0338': 'nbumpe',
        '\u2250': 'doteq',
        '\u2250\u0338': 'nedot',
        '\u2251': 'eDot',
        '\u2252': 'efDot',
        '\u2253': 'erDot',
        '\u2254': 'colone',
        '\u2255': 'ecolon',
        '\u2256': 'ecir',
        '\u2257': 'cire',
        '\u2259': 'wedgeq',
        '\u225A': 'veeeq',
        '\u225C': 'trie',
        '\u225F': 'equest',
        '\u2261': 'equiv',
        '\u2262': 'nequiv',
        '\u2261\u20E5': 'bnequiv',
        '\u2264': 'le',
        '\u2270': 'nle',
        '\u2264\u20D2': 'nvle',
        '\u2265': 'ge',
        '\u2271': 'nge',
        '\u2265\u20D2': 'nvge',
        '\u2266': 'lE',
        '\u2266\u0338': 'nlE',
        '\u2267': 'gE',
        '\u2267\u0338': 'ngE',
        '\u2268\uFE00': 'lvnE',
        '\u2268': 'lnE',
        '\u2269': 'gnE',
        '\u2269\uFE00': 'gvnE',
        '\u226A': 'll',
        '\u226A\u0338': 'nLtv',
        '\u226A\u20D2': 'nLt',
        '\u226B': 'gg',
        '\u226B\u0338': 'nGtv',
        '\u226B\u20D2': 'nGt',
        '\u226C': 'twixt',
        '\u2272': 'lsim',
        '\u2274': 'nlsim',
        '\u2273': 'gsim',
        '\u2275': 'ngsim',
        '\u2276': 'lg',
        '\u2278': 'ntlg',
        '\u2277': 'gl',
        '\u2279': 'ntgl',
        '\u227A': 'pr',
        '\u2280': 'npr',
        '\u227B': 'sc',
        '\u2281': 'nsc',
        '\u227C': 'prcue',
        '\u22E0': 'nprcue',
        '\u227D': 'sccue',
        '\u22E1': 'nsccue',
        '\u227E': 'prsim',
        '\u227F': 'scsim',
        '\u227F\u0338': 'NotSucceedsTilde',
        '\u2282': 'sub',
        '\u2284': 'nsub',
        '\u2282\u20D2': 'vnsub',
        '\u2283': 'sup',
        '\u2285': 'nsup',
        '\u2283\u20D2': 'vnsup',
        '\u2286': 'sube',
        '\u2288': 'nsube',
        '\u2287': 'supe',
        '\u2289': 'nsupe',
        '\u228A\uFE00': 'vsubne',
        '\u228A': 'subne',
        '\u228B\uFE00': 'vsupne',
        '\u228B': 'supne',
        '\u228D': 'cupdot',
        '\u228E': 'uplus',
        '\u228F': 'sqsub',
        '\u228F\u0338': 'NotSquareSubset',
        '\u2290': 'sqsup',
        '\u2290\u0338': 'NotSquareSuperset',
        '\u2291': 'sqsube',
        '\u22E2': 'nsqsube',
        '\u2292': 'sqsupe',
        '\u22E3': 'nsqsupe',
        '\u2293': 'sqcap',
        '\u2293\uFE00': 'sqcaps',
        '\u2294': 'sqcup',
        '\u2294\uFE00': 'sqcups',
        '\u2295': 'oplus',
        '\u2296': 'ominus',
        '\u2297': 'otimes',
        '\u2298': 'osol',
        '\u2299': 'odot',
        '\u229A': 'ocir',
        '\u229B': 'oast',
        '\u229D': 'odash',
        '\u229E': 'plusb',
        '\u229F': 'minusb',
        '\u22A0': 'timesb',
        '\u22A1': 'sdotb',
        '\u22A2': 'vdash',
        '\u22AC': 'nvdash',
        '\u22A3': 'dashv',
        '\u22A4': 'top',
        '\u22A5': 'bot',
        '\u22A7': 'models',
        '\u22A8': 'vDash',
        '\u22AD': 'nvDash',
        '\u22A9': 'Vdash',
        '\u22AE': 'nVdash',
        '\u22AA': 'Vvdash',
        '\u22AB': 'VDash',
        '\u22AF': 'nVDash',
        '\u22B0': 'prurel',
        '\u22B2': 'vltri',
        '\u22EA': 'nltri',
        '\u22B3': 'vrtri',
        '\u22EB': 'nrtri',
        '\u22B4': 'ltrie',
        '\u22EC': 'nltrie',
        '\u22B4\u20D2': 'nvltrie',
        '\u22B5': 'rtrie',
        '\u22ED': 'nrtrie',
        '\u22B5\u20D2': 'nvrtrie',
        '\u22B6': 'origof',
        '\u22B7': 'imof',
        '\u22B8': 'mumap',
        '\u22B9': 'hercon',
        '\u22BA': 'intcal',
        '\u22BB': 'veebar',
        '\u22BD': 'barvee',
        '\u22BE': 'angrtvb',
        '\u22BF': 'lrtri',
        '\u22C0': 'Wedge',
        '\u22C1': 'Vee',
        '\u22C2': 'xcap',
        '\u22C3': 'xcup',
        '\u22C4': 'diam',
        '\u22C5': 'sdot',
        '\u22C6': 'Star',
        '\u22C7': 'divonx',
        '\u22C8': 'bowtie',
        '\u22C9': 'ltimes',
        '\u22CA': 'rtimes',
        '\u22CB': 'lthree',
        '\u22CC': 'rthree',
        '\u22CD': 'bsime',
        '\u22CE': 'cuvee',
        '\u22CF': 'cuwed',
        '\u22D0': 'Sub',
        '\u22D1': 'Sup',
        '\u22D2': 'Cap',
        '\u22D3': 'Cup',
        '\u22D4': 'fork',
        '\u22D5': 'epar',
        '\u22D6': 'ltdot',
        '\u22D7': 'gtdot',
        '\u22D8': 'Ll',
        '\u22D8\u0338': 'nLl',
        '\u22D9': 'Gg',
        '\u22D9\u0338': 'nGg',
        '\u22DA\uFE00': 'lesg',
        '\u22DA': 'leg',
        '\u22DB': 'gel',
        '\u22DB\uFE00': 'gesl',
        '\u22DE': 'cuepr',
        '\u22DF': 'cuesc',
        '\u22E6': 'lnsim',
        '\u22E7': 'gnsim',
        '\u22E8': 'prnsim',
        '\u22E9': 'scnsim',
        '\u22EE': 'vellip',
        '\u22EF': 'ctdot',
        '\u22F0': 'utdot',
        '\u22F1': 'dtdot',
        '\u22F2': 'disin',
        '\u22F3': 'isinsv',
        '\u22F4': 'isins',
        '\u22F5': 'isindot',
        '\u22F5\u0338': 'notindot',
        '\u22F6': 'notinvc',
        '\u22F7': 'notinvb',
        '\u22F9': 'isinE',
        '\u22F9\u0338': 'notinE',
        '\u22FA': 'nisd',
        '\u22FB': 'xnis',
        '\u22FC': 'nis',
        '\u22FD': 'notnivc',
        '\u22FE': 'notnivb',
        '\u2305': 'barwed',
        '\u2306': 'Barwed',
        '\u230C': 'drcrop',
        '\u230D': 'dlcrop',
        '\u230E': 'urcrop',
        '\u230F': 'ulcrop',
        '\u2310': 'bnot',
        '\u2312': 'profline',
        '\u2313': 'profsurf',
        '\u2315': 'telrec',
        '\u2316': 'target',
        '\u231C': 'ulcorn',
        '\u231D': 'urcorn',
        '\u231E': 'dlcorn',
        '\u231F': 'drcorn',
        '\u2322': 'frown',
        '\u2323': 'smile',
        '\u232D': 'cylcty',
        '\u232E': 'profalar',
        '\u2336': 'topbot',
        '\u233D': 'ovbar',
        '\u233F': 'solbar',
        '\u237C': 'angzarr',
        '\u23B0': 'lmoust',
        '\u23B1': 'rmoust',
        '\u23B4': 'tbrk',
        '\u23B5': 'bbrk',
        '\u23B6': 'bbrktbrk',
        '\u23DC': 'OverParenthesis',
        '\u23DD': 'UnderParenthesis',
        '\u23DE': 'OverBrace',
        '\u23DF': 'UnderBrace',
        '\u23E2': 'trpezium',
        '\u23E7': 'elinters',
        '\u2423': 'blank',
        '\u2500': 'boxh',
        '\u2502': 'boxv',
        '\u250C': 'boxdr',
        '\u2510': 'boxdl',
        '\u2514': 'boxur',
        '\u2518': 'boxul',
        '\u251C': 'boxvr',
        '\u2524': 'boxvl',
        '\u252C': 'boxhd',
        '\u2534': 'boxhu',
        '\u253C': 'boxvh',
        '\u2550': 'boxH',
        '\u2551': 'boxV',
        '\u2552': 'boxdR',
        '\u2553': 'boxDr',
        '\u2554': 'boxDR',
        '\u2555': 'boxdL',
        '\u2556': 'boxDl',
        '\u2557': 'boxDL',
        '\u2558': 'boxuR',
        '\u2559': 'boxUr',
        '\u255A': 'boxUR',
        '\u255B': 'boxuL',
        '\u255C': 'boxUl',
        '\u255D': 'boxUL',
        '\u255E': 'boxvR',
        '\u255F': 'boxVr',
        '\u2560': 'boxVR',
        '\u2561': 'boxvL',
        '\u2562': 'boxVl',
        '\u2563': 'boxVL',
        '\u2564': 'boxHd',
        '\u2565': 'boxhD',
        '\u2566': 'boxHD',
        '\u2567': 'boxHu',
        '\u2568': 'boxhU',
        '\u2569': 'boxHU',
        '\u256A': 'boxvH',
        '\u256B': 'boxVh',
        '\u256C': 'boxVH',
        '\u2580': 'uhblk',
        '\u2584': 'lhblk',
        '\u2588': 'block',
        '\u2591': 'blk14',
        '\u2592': 'blk12',
        '\u2593': 'blk34',
        '\u25A1': 'squ',
        '\u25AA': 'squf',
        '\u25AB': 'EmptyVerySmallSquare',
        '\u25AD': 'rect',
        '\u25AE': 'marker',
        '\u25B1': 'fltns',
        '\u25B3': 'xutri',
        '\u25B4': 'utrif',
        '\u25B5': 'utri',
        '\u25B8': 'rtrif',
        '\u25B9': 'rtri',
        '\u25BD': 'xdtri',
        '\u25BE': 'dtrif',
        '\u25BF': 'dtri',
        '\u25C2': 'ltrif',
        '\u25C3': 'ltri',
        '\u25CA': 'loz',
        '\u25CB': 'cir',
        '\u25EC': 'tridot',
        '\u25EF': 'xcirc',
        '\u25F8': 'ultri',
        '\u25F9': 'urtri',
        '\u25FA': 'lltri',
        '\u25FB': 'EmptySmallSquare',
        '\u25FC': 'FilledSmallSquare',
        '\u2605': 'starf',
        '\u2606': 'star',
        '\u260E': 'phone',
        '\u2640': 'female',
        '\u2642': 'male',
        '\u2660': 'spades',
        '\u2663': 'clubs',
        '\u2665': 'hearts',
        '\u2666': 'diams',
        '\u266A': 'sung',
        '\u2713': 'check',
        '\u2717': 'cross',
        '\u2720': 'malt',
        '\u2736': 'sext',
        '\u2758': 'VerticalSeparator',
        '\u27C8': 'bsolhsub',
        '\u27C9': 'suphsol',
        '\u27F5': 'xlarr',
        '\u27F6': 'xrarr',
        '\u27F7': 'xharr',
        '\u27F8': 'xlArr',
        '\u27F9': 'xrArr',
        '\u27FA': 'xhArr',
        '\u27FC': 'xmap',
        '\u27FF': 'dzigrarr',
        '\u2902': 'nvlArr',
        '\u2903': 'nvrArr',
        '\u2904': 'nvHarr',
        '\u2905': 'Map',
        '\u290C': 'lbarr',
        '\u290D': 'rbarr',
        '\u290E': 'lBarr',
        '\u290F': 'rBarr',
        '\u2910': 'RBarr',
        '\u2911': 'DDotrahd',
        '\u2912': 'UpArrowBar',
        '\u2913': 'DownArrowBar',
        '\u2916': 'Rarrtl',
        '\u2919': 'latail',
        '\u291A': 'ratail',
        '\u291B': 'lAtail',
        '\u291C': 'rAtail',
        '\u291D': 'larrfs',
        '\u291E': 'rarrfs',
        '\u291F': 'larrbfs',
        '\u2920': 'rarrbfs',
        '\u2923': 'nwarhk',
        '\u2924': 'nearhk',
        '\u2925': 'searhk',
        '\u2926': 'swarhk',
        '\u2927': 'nwnear',
        '\u2928': 'toea',
        '\u2929': 'tosa',
        '\u292A': 'swnwar',
        '\u2933': 'rarrc',
        '\u2933\u0338': 'nrarrc',
        '\u2935': 'cudarrr',
        '\u2936': 'ldca',
        '\u2937': 'rdca',
        '\u2938': 'cudarrl',
        '\u2939': 'larrpl',
        '\u293C': 'curarrm',
        '\u293D': 'cularrp',
        '\u2945': 'rarrpl',
        '\u2948': 'harrcir',
        '\u2949': 'Uarrocir',
        '\u294A': 'lurdshar',
        '\u294B': 'ldrushar',
        '\u294E': 'LeftRightVector',
        '\u294F': 'RightUpDownVector',
        '\u2950': 'DownLeftRightVector',
        '\u2951': 'LeftUpDownVector',
        '\u2952': 'LeftVectorBar',
        '\u2953': 'RightVectorBar',
        '\u2954': 'RightUpVectorBar',
        '\u2955': 'RightDownVectorBar',
        '\u2956': 'DownLeftVectorBar',
        '\u2957': 'DownRightVectorBar',
        '\u2958': 'LeftUpVectorBar',
        '\u2959': 'LeftDownVectorBar',
        '\u295A': 'LeftTeeVector',
        '\u295B': 'RightTeeVector',
        '\u295C': 'RightUpTeeVector',
        '\u295D': 'RightDownTeeVector',
        '\u295E': 'DownLeftTeeVector',
        '\u295F': 'DownRightTeeVector',
        '\u2960': 'LeftUpTeeVector',
        '\u2961': 'LeftDownTeeVector',
        '\u2962': 'lHar',
        '\u2963': 'uHar',
        '\u2964': 'rHar',
        '\u2965': 'dHar',
        '\u2966': 'luruhar',
        '\u2967': 'ldrdhar',
        '\u2968': 'ruluhar',
        '\u2969': 'rdldhar',
        '\u296A': 'lharul',
        '\u296B': 'llhard',
        '\u296C': 'rharul',
        '\u296D': 'lrhard',
        '\u296E': 'udhar',
        '\u296F': 'duhar',
        '\u2970': 'RoundImplies',
        '\u2971': 'erarr',
        '\u2972': 'simrarr',
        '\u2973': 'larrsim',
        '\u2974': 'rarrsim',
        '\u2975': 'rarrap',
        '\u2976': 'ltlarr',
        '\u2978': 'gtrarr',
        '\u2979': 'subrarr',
        '\u297B': 'suplarr',
        '\u297C': 'lfisht',
        '\u297D': 'rfisht',
        '\u297E': 'ufisht',
        '\u297F': 'dfisht',
        '\u299A': 'vzigzag',
        '\u299C': 'vangrt',
        '\u299D': 'angrtvbd',
        '\u29A4': 'ange',
        '\u29A5': 'range',
        '\u29A6': 'dwangle',
        '\u29A7': 'uwangle',
        '\u29A8': 'angmsdaa',
        '\u29A9': 'angmsdab',
        '\u29AA': 'angmsdac',
        '\u29AB': 'angmsdad',
        '\u29AC': 'angmsdae',
        '\u29AD': 'angmsdaf',
        '\u29AE': 'angmsdag',
        '\u29AF': 'angmsdah',
        '\u29B0': 'bemptyv',
        '\u29B1': 'demptyv',
        '\u29B2': 'cemptyv',
        '\u29B3': 'raemptyv',
        '\u29B4': 'laemptyv',
        '\u29B5': 'ohbar',
        '\u29B6': 'omid',
        '\u29B7': 'opar',
        '\u29B9': 'operp',
        '\u29BB': 'olcross',
        '\u29BC': 'odsold',
        '\u29BE': 'olcir',
        '\u29BF': 'ofcir',
        '\u29C0': 'olt',
        '\u29C1': 'ogt',
        '\u29C2': 'cirscir',
        '\u29C3': 'cirE',
        '\u29C4': 'solb',
        '\u29C5': 'bsolb',
        '\u29C9': 'boxbox',
        '\u29CD': 'trisb',
        '\u29CE': 'rtriltri',
        '\u29CF': 'LeftTriangleBar',
        '\u29CF\u0338': 'NotLeftTriangleBar',
        '\u29D0': 'RightTriangleBar',
        '\u29D0\u0338': 'NotRightTriangleBar',
        '\u29DC': 'iinfin',
        '\u29DD': 'infintie',
        '\u29DE': 'nvinfin',
        '\u29E3': 'eparsl',
        '\u29E4': 'smeparsl',
        '\u29E5': 'eqvparsl',
        '\u29EB': 'lozf',
        '\u29F4': 'RuleDelayed',
        '\u29F6': 'dsol',
        '\u2A00': 'xodot',
        '\u2A01': 'xoplus',
        '\u2A02': 'xotime',
        '\u2A04': 'xuplus',
        '\u2A06': 'xsqcup',
        '\u2A0D': 'fpartint',
        '\u2A10': 'cirfnint',
        '\u2A11': 'awint',
        '\u2A12': 'rppolint',
        '\u2A13': 'scpolint',
        '\u2A14': 'npolint',
        '\u2A15': 'pointint',
        '\u2A16': 'quatint',
        '\u2A17': 'intlarhk',
        '\u2A22': 'pluscir',
        '\u2A23': 'plusacir',
        '\u2A24': 'simplus',
        '\u2A25': 'plusdu',
        '\u2A26': 'plussim',
        '\u2A27': 'plustwo',
        '\u2A29': 'mcomma',
        '\u2A2A': 'minusdu',
        '\u2A2D': 'loplus',
        '\u2A2E': 'roplus',
        '\u2A2F': 'Cross',
        '\u2A30': 'timesd',
        '\u2A31': 'timesbar',
        '\u2A33': 'smashp',
        '\u2A34': 'lotimes',
        '\u2A35': 'rotimes',
        '\u2A36': 'otimesas',
        '\u2A37': 'Otimes',
        '\u2A38': 'odiv',
        '\u2A39': 'triplus',
        '\u2A3A': 'triminus',
        '\u2A3B': 'tritime',
        '\u2A3C': 'iprod',
        '\u2A3F': 'amalg',
        '\u2A40': 'capdot',
        '\u2A42': 'ncup',
        '\u2A43': 'ncap',
        '\u2A44': 'capand',
        '\u2A45': 'cupor',
        '\u2A46': 'cupcap',
        '\u2A47': 'capcup',
        '\u2A48': 'cupbrcap',
        '\u2A49': 'capbrcup',
        '\u2A4A': 'cupcup',
        '\u2A4B': 'capcap',
        '\u2A4C': 'ccups',
        '\u2A4D': 'ccaps',
        '\u2A50': 'ccupssm',
        '\u2A53': 'And',
        '\u2A54': 'Or',
        '\u2A55': 'andand',
        '\u2A56': 'oror',
        '\u2A57': 'orslope',
        '\u2A58': 'andslope',
        '\u2A5A': 'andv',
        '\u2A5B': 'orv',
        '\u2A5C': 'andd',
        '\u2A5D': 'ord',
        '\u2A5F': 'wedbar',
        '\u2A66': 'sdote',
        '\u2A6A': 'simdot',
        '\u2A6D': 'congdot',
        '\u2A6D\u0338': 'ncongdot',
        '\u2A6E': 'easter',
        '\u2A6F': 'apacir',
        '\u2A70': 'apE',
        '\u2A70\u0338': 'napE',
        '\u2A71': 'eplus',
        '\u2A72': 'pluse',
        '\u2A73': 'Esim',
        '\u2A77': 'eDDot',
        '\u2A78': 'equivDD',
        '\u2A79': 'ltcir',
        '\u2A7A': 'gtcir',
        '\u2A7B': 'ltquest',
        '\u2A7C': 'gtquest',
        '\u2A7D': 'les',
        '\u2A7D\u0338': 'nles',
        '\u2A7E': 'ges',
        '\u2A7E\u0338': 'nges',
        '\u2A7F': 'lesdot',
        '\u2A80': 'gesdot',
        '\u2A81': 'lesdoto',
        '\u2A82': 'gesdoto',
        '\u2A83': 'lesdotor',
        '\u2A84': 'gesdotol',
        '\u2A85': 'lap',
        '\u2A86': 'gap',
        '\u2A87': 'lne',
        '\u2A88': 'gne',
        '\u2A89': 'lnap',
        '\u2A8A': 'gnap',
        '\u2A8B': 'lEg',
        '\u2A8C': 'gEl',
        '\u2A8D': 'lsime',
        '\u2A8E': 'gsime',
        '\u2A8F': 'lsimg',
        '\u2A90': 'gsiml',
        '\u2A91': 'lgE',
        '\u2A92': 'glE',
        '\u2A93': 'lesges',
        '\u2A94': 'gesles',
        '\u2A95': 'els',
        '\u2A96': 'egs',
        '\u2A97': 'elsdot',
        '\u2A98': 'egsdot',
        '\u2A99': 'el',
        '\u2A9A': 'eg',
        '\u2A9D': 'siml',
        '\u2A9E': 'simg',
        '\u2A9F': 'simlE',
        '\u2AA0': 'simgE',
        '\u2AA1': 'LessLess',
        '\u2AA1\u0338': 'NotNestedLessLess',
        '\u2AA2': 'GreaterGreater',
        '\u2AA2\u0338': 'NotNestedGreaterGreater',
        '\u2AA4': 'glj',
        '\u2AA5': 'gla',
        '\u2AA6': 'ltcc',
        '\u2AA7': 'gtcc',
        '\u2AA8': 'lescc',
        '\u2AA9': 'gescc',
        '\u2AAA': 'smt',
        '\u2AAB': 'lat',
        '\u2AAC': 'smte',
        '\u2AAC\uFE00': 'smtes',
        '\u2AAD': 'late',
        '\u2AAD\uFE00': 'lates',
        '\u2AAE': 'bumpE',
        '\u2AAF': 'pre',
        '\u2AAF\u0338': 'npre',
        '\u2AB0': 'sce',
        '\u2AB0\u0338': 'nsce',
        '\u2AB3': 'prE',
        '\u2AB4': 'scE',
        '\u2AB5': 'prnE',
        '\u2AB6': 'scnE',
        '\u2AB7': 'prap',
        '\u2AB8': 'scap',
        '\u2AB9': 'prnap',
        '\u2ABA': 'scnap',
        '\u2ABB': 'Pr',
        '\u2ABC': 'Sc',
        '\u2ABD': 'subdot',
        '\u2ABE': 'supdot',
        '\u2ABF': 'subplus',
        '\u2AC0': 'supplus',
        '\u2AC1': 'submult',
        '\u2AC2': 'supmult',
        '\u2AC3': 'subedot',
        '\u2AC4': 'supedot',
        '\u2AC5': 'subE',
        '\u2AC5\u0338': 'nsubE',
        '\u2AC6': 'supE',
        '\u2AC6\u0338': 'nsupE',
        '\u2AC7': 'subsim',
        '\u2AC8': 'supsim',
        '\u2ACB\uFE00': 'vsubnE',
        '\u2ACB': 'subnE',
        '\u2ACC\uFE00': 'vsupnE',
        '\u2ACC': 'supnE',
        '\u2ACF': 'csub',
        '\u2AD0': 'csup',
        '\u2AD1': 'csube',
        '\u2AD2': 'csupe',
        '\u2AD3': 'subsup',
        '\u2AD4': 'supsub',
        '\u2AD5': 'subsub',
        '\u2AD6': 'supsup',
        '\u2AD7': 'suphsub',
        '\u2AD8': 'supdsub',
        '\u2AD9': 'forkv',
        '\u2ADA': 'topfork',
        '\u2ADB': 'mlcp',
        '\u2AE4': 'Dashv',
        '\u2AE6': 'Vdashl',
        '\u2AE7': 'Barv',
        '\u2AE8': 'vBar',
        '\u2AE9': 'vBarv',
        '\u2AEB': 'Vbar',
        '\u2AEC': 'Not',
        '\u2AED': 'bNot',
        '\u2AEE': 'rnmid',
        '\u2AEF': 'cirmid',
        '\u2AF0': 'midcir',
        '\u2AF1': 'topcir',
        '\u2AF2': 'nhpar',
        '\u2AF3': 'parsim',
        '\u2AFD': 'parsl',
        '\u2AFD\u20E5': 'nparsl',
        '\u266D': 'flat',
        '\u266E': 'natur',
        '\u266F': 'sharp',
        '\xA4': 'curren',
        '\xA2': 'cent',
        '$': 'dollar',
        '\xA3': 'pound',
        '\xA5': 'yen',
        '\u20AC': 'euro',
        '\xB9': 'sup1',
        '\xBD': 'half',
        '\u2153': 'frac13',
        '\xBC': 'frac14',
        '\u2155': 'frac15',
        '\u2159': 'frac16',
        '\u215B': 'frac18',
        '\xB2': 'sup2',
        '\u2154': 'frac23',
        '\u2156': 'frac25',
        '\xB3': 'sup3',
        '\xBE': 'frac34',
        '\u2157': 'frac35',
        '\u215C': 'frac38',
        '\u2158': 'frac45',
        '\u215A': 'frac56',
        '\u215D': 'frac58',
        '\u215E': 'frac78',
        '\uD835\uDCB6': 'ascr',
        '\uD835\uDD52': 'aopf',
        '\uD835\uDD1E': 'afr',
        '\uD835\uDD38': 'Aopf',
        '\uD835\uDD04': 'Afr',
        '\uD835\uDC9C': 'Ascr',
        '\xAA': 'ordf',
        '\xE1': 'aacute',
        '\xC1': 'Aacute',
        '\xE0': 'agrave',
        '\xC0': 'Agrave',
        '\u0103': 'abreve',
        '\u0102': 'Abreve',
        '\xE2': 'acirc',
        '\xC2': 'Acirc',
        '\xE5': 'aring',
        '\xC5': 'angst',
        '\xE4': 'auml',
        '\xC4': 'Auml',
        '\xE3': 'atilde',
        '\xC3': 'Atilde',
        '\u0105': 'aogon',
        '\u0104': 'Aogon',
        '\u0101': 'amacr',
        '\u0100': 'Amacr',
        '\xE6': 'aelig',
        '\xC6': 'AElig',
        '\uD835\uDCB7': 'bscr',
        '\uD835\uDD53': 'bopf',
        '\uD835\uDD1F': 'bfr',
        '\uD835\uDD39': 'Bopf',
        '\u212C': 'Bscr',
        '\uD835\uDD05': 'Bfr',
        '\uD835\uDD20': 'cfr',
        '\uD835\uDCB8': 'cscr',
        '\uD835\uDD54': 'copf',
        '\u212D': 'Cfr',
        '\uD835\uDC9E': 'Cscr',
        '\u2102': 'Copf',
        '\u0107': 'cacute',
        '\u0106': 'Cacute',
        '\u0109': 'ccirc',
        '\u0108': 'Ccirc',
        '\u010D': 'ccaron',
        '\u010C': 'Ccaron',
        '\u010B': 'cdot',
        '\u010A': 'Cdot',
        '\xE7': 'ccedil',
        '\xC7': 'Ccedil',
        '\u2105': 'incare',
        '\uD835\uDD21': 'dfr',
        '\u2146': 'dd',
        '\uD835\uDD55': 'dopf',
        '\uD835\uDCB9': 'dscr',
        '\uD835\uDC9F': 'Dscr',
        '\uD835\uDD07': 'Dfr',
        '\u2145': 'DD',
        '\uD835\uDD3B': 'Dopf',
        '\u010F': 'dcaron',
        '\u010E': 'Dcaron',
        '\u0111': 'dstrok',
        '\u0110': 'Dstrok',
        '\xF0': 'eth',
        '\xD0': 'ETH',
        '\u2147': 'ee',
        '\u212F': 'escr',
        '\uD835\uDD22': 'efr',
        '\uD835\uDD56': 'eopf',
        '\u2130': 'Escr',
        '\uD835\uDD08': 'Efr',
        '\uD835\uDD3C': 'Eopf',
        '\xE9': 'eacute',
        '\xC9': 'Eacute',
        '\xE8': 'egrave',
        '\xC8': 'Egrave',
        '\xEA': 'ecirc',
        '\xCA': 'Ecirc',
        '\u011B': 'ecaron',
        '\u011A': 'Ecaron',
        '\xEB': 'euml',
        '\xCB': 'Euml',
        '\u0117': 'edot',
        '\u0116': 'Edot',
        '\u0119': 'eogon',
        '\u0118': 'Eogon',
        '\u0113': 'emacr',
        '\u0112': 'Emacr',
        '\uD835\uDD23': 'ffr',
        '\uD835\uDD57': 'fopf',
        '\uD835\uDCBB': 'fscr',
        '\uD835\uDD09': 'Ffr',
        '\uD835\uDD3D': 'Fopf',
        '\u2131': 'Fscr',
        '\uFB00': 'fflig',
        '\uFB03': 'ffilig',
        '\uFB04': 'ffllig',
        '\uFB01': 'filig',
        'fj': 'fjlig',
        '\uFB02': 'fllig',
        '\u0192': 'fnof',
        '\u210A': 'gscr',
        '\uD835\uDD58': 'gopf',
        '\uD835\uDD24': 'gfr',
        '\uD835\uDCA2': 'Gscr',
        '\uD835\uDD3E': 'Gopf',
        '\uD835\uDD0A': 'Gfr',
        '\u01F5': 'gacute',
        '\u011F': 'gbreve',
        '\u011E': 'Gbreve',
        '\u011D': 'gcirc',
        '\u011C': 'Gcirc',
        '\u0121': 'gdot',
        '\u0120': 'Gdot',
        '\u0122': 'Gcedil',
        '\uD835\uDD25': 'hfr',
        '\u210E': 'planckh',
        '\uD835\uDCBD': 'hscr',
        '\uD835\uDD59': 'hopf',
        '\u210B': 'Hscr',
        '\u210C': 'Hfr',
        '\u210D': 'Hopf',
        '\u0125': 'hcirc',
        '\u0124': 'Hcirc',
        '\u210F': 'hbar',
        '\u0127': 'hstrok',
        '\u0126': 'Hstrok',
        '\uD835\uDD5A': 'iopf',
        '\uD835\uDD26': 'ifr',
        '\uD835\uDCBE': 'iscr',
        '\u2148': 'ii',
        '\uD835\uDD40': 'Iopf',
        '\u2110': 'Iscr',
        '\u2111': 'Im',
        '\xED': 'iacute',
        '\xCD': 'Iacute',
        '\xEC': 'igrave',
        '\xCC': 'Igrave',
        '\xEE': 'icirc',
        '\xCE': 'Icirc',
        '\xEF': 'iuml',
        '\xCF': 'Iuml',
        '\u0129': 'itilde',
        '\u0128': 'Itilde',
        '\u0130': 'Idot',
        '\u012F': 'iogon',
        '\u012E': 'Iogon',
        '\u012B': 'imacr',
        '\u012A': 'Imacr',
        '\u0133': 'ijlig',
        '\u0132': 'IJlig',
        '\u0131': 'imath',
        '\uD835\uDCBF': 'jscr',
        '\uD835\uDD5B': 'jopf',
        '\uD835\uDD27': 'jfr',
        '\uD835\uDCA5': 'Jscr',
        '\uD835\uDD0D': 'Jfr',
        '\uD835\uDD41': 'Jopf',
        '\u0135': 'jcirc',
        '\u0134': 'Jcirc',
        '\u0237': 'jmath',
        '\uD835\uDD5C': 'kopf',
        '\uD835\uDCC0': 'kscr',
        '\uD835\uDD28': 'kfr',
        '\uD835\uDCA6': 'Kscr',
        '\uD835\uDD42': 'Kopf',
        '\uD835\uDD0E': 'Kfr',
        '\u0137': 'kcedil',
        '\u0136': 'Kcedil',
        '\uD835\uDD29': 'lfr',
        '\uD835\uDCC1': 'lscr',
        '\u2113': 'ell',
        '\uD835\uDD5D': 'lopf',
        '\u2112': 'Lscr',
        '\uD835\uDD0F': 'Lfr',
        '\uD835\uDD43': 'Lopf',
        '\u013A': 'lacute',
        '\u0139': 'Lacute',
        '\u013E': 'lcaron',
        '\u013D': 'Lcaron',
        '\u013C': 'lcedil',
        '\u013B': 'Lcedil',
        '\u0142': 'lstrok',
        '\u0141': 'Lstrok',
        '\u0140': 'lmidot',
        '\u013F': 'Lmidot',
        '\uD835\uDD2A': 'mfr',
        '\uD835\uDD5E': 'mopf',
        '\uD835\uDCC2': 'mscr',
        '\uD835\uDD10': 'Mfr',
        '\uD835\uDD44': 'Mopf',
        '\u2133': 'Mscr',
        '\uD835\uDD2B': 'nfr',
        '\uD835\uDD5F': 'nopf',
        '\uD835\uDCC3': 'nscr',
        '\u2115': 'Nopf',
        '\uD835\uDCA9': 'Nscr',
        '\uD835\uDD11': 'Nfr',
        '\u0144': 'nacute',
        '\u0143': 'Nacute',
        '\u0148': 'ncaron',
        '\u0147': 'Ncaron',
        '\xF1': 'ntilde',
        '\xD1': 'Ntilde',
        '\u0146': 'ncedil',
        '\u0145': 'Ncedil',
        '\u2116': 'numero',
        '\u014B': 'eng',
        '\u014A': 'ENG',
        '\uD835\uDD60': 'oopf',
        '\uD835\uDD2C': 'ofr',
        '\u2134': 'oscr',
        '\uD835\uDCAA': 'Oscr',
        '\uD835\uDD12': 'Ofr',
        '\uD835\uDD46': 'Oopf',
        '\xBA': 'ordm',
        '\xF3': 'oacute',
        '\xD3': 'Oacute',
        '\xF2': 'ograve',
        '\xD2': 'Ograve',
        '\xF4': 'ocirc',
        '\xD4': 'Ocirc',
        '\xF6': 'ouml',
        '\xD6': 'Ouml',
        '\u0151': 'odblac',
        '\u0150': 'Odblac',
        '\xF5': 'otilde',
        '\xD5': 'Otilde',
        '\xF8': 'oslash',
        '\xD8': 'Oslash',
        '\u014D': 'omacr',
        '\u014C': 'Omacr',
        '\u0153': 'oelig',
        '\u0152': 'OElig',
        '\uD835\uDD2D': 'pfr',
        '\uD835\uDCC5': 'pscr',
        '\uD835\uDD61': 'popf',
        '\u2119': 'Popf',
        '\uD835\uDD13': 'Pfr',
        '\uD835\uDCAB': 'Pscr',
        '\uD835\uDD62': 'qopf',
        '\uD835\uDD2E': 'qfr',
        '\uD835\uDCC6': 'qscr',
        '\uD835\uDCAC': 'Qscr',
        '\uD835\uDD14': 'Qfr',
        '\u211A': 'Qopf',
        '\u0138': 'kgreen',
        '\uD835\uDD2F': 'rfr',
        '\uD835\uDD63': 'ropf',
        '\uD835\uDCC7': 'rscr',
        '\u211B': 'Rscr',
        '\u211C': 'Re',
        '\u211D': 'Ropf',
        '\u0155': 'racute',
        '\u0154': 'Racute',
        '\u0159': 'rcaron',
        '\u0158': 'Rcaron',
        '\u0157': 'rcedil',
        '\u0156': 'Rcedil',
        '\uD835\uDD64': 'sopf',
        '\uD835\uDCC8': 'sscr',
        '\uD835\uDD30': 'sfr',
        '\uD835\uDD4A': 'Sopf',
        '\uD835\uDD16': 'Sfr',
        '\uD835\uDCAE': 'Sscr',
        '\u24C8': 'oS',
        '\u015B': 'sacute',
        '\u015A': 'Sacute',
        '\u015D': 'scirc',
        '\u015C': 'Scirc',
        '\u0161': 'scaron',
        '\u0160': 'Scaron',
        '\u015F': 'scedil',
        '\u015E': 'Scedil',
        '\xDF': 'szlig',
        '\uD835\uDD31': 'tfr',
        '\uD835\uDCC9': 'tscr',
        '\uD835\uDD65': 'topf',
        '\uD835\uDCAF': 'Tscr',
        '\uD835\uDD17': 'Tfr',
        '\uD835\uDD4B': 'Topf',
        '\u0165': 'tcaron',
        '\u0164': 'Tcaron',
        '\u0163': 'tcedil',
        '\u0162': 'Tcedil',
        '\u2122': 'trade',
        '\u0167': 'tstrok',
        '\u0166': 'Tstrok',
        '\uD835\uDCCA': 'uscr',
        '\uD835\uDD66': 'uopf',
        '\uD835\uDD32': 'ufr',
        '\uD835\uDD4C': 'Uopf',
        '\uD835\uDD18': 'Ufr',
        '\uD835\uDCB0': 'Uscr',
        '\xFA': 'uacute',
        '\xDA': 'Uacute',
        '\xF9': 'ugrave',
        '\xD9': 'Ugrave',
        '\u016D': 'ubreve',
        '\u016C': 'Ubreve',
        '\xFB': 'ucirc',
        '\xDB': 'Ucirc',
        '\u016F': 'uring',
        '\u016E': 'Uring',
        '\xFC': 'uuml',
        '\xDC': 'Uuml',
        '\u0171': 'udblac',
        '\u0170': 'Udblac',
        '\u0169': 'utilde',
        '\u0168': 'Utilde',
        '\u0173': 'uogon',
        '\u0172': 'Uogon',
        '\u016B': 'umacr',
        '\u016A': 'Umacr',
        '\uD835\uDD33': 'vfr',
        '\uD835\uDD67': 'vopf',
        '\uD835\uDCCB': 'vscr',
        '\uD835\uDD19': 'Vfr',
        '\uD835\uDD4D': 'Vopf',
        '\uD835\uDCB1': 'Vscr',
        '\uD835\uDD68': 'wopf',
        '\uD835\uDCCC': 'wscr',
        '\uD835\uDD34': 'wfr',
        '\uD835\uDCB2': 'Wscr',
        '\uD835\uDD4E': 'Wopf',
        '\uD835\uDD1A': 'Wfr',
        '\u0175': 'wcirc',
        '\u0174': 'Wcirc',
        '\uD835\uDD35': 'xfr',
        '\uD835\uDCCD': 'xscr',
        '\uD835\uDD69': 'xopf',
        '\uD835\uDD4F': 'Xopf',
        '\uD835\uDD1B': 'Xfr',
        '\uD835\uDCB3': 'Xscr',
        '\uD835\uDD36': 'yfr',
        '\uD835\uDCCE': 'yscr',
        '\uD835\uDD6A': 'yopf',
        '\uD835\uDCB4': 'Yscr',
        '\uD835\uDD1C': 'Yfr',
        '\uD835\uDD50': 'Yopf',
        '\xFD': 'yacute',
        '\xDD': 'Yacute',
        '\u0177': 'ycirc',
        '\u0176': 'Ycirc',
        '\xFF': 'yuml',
        '\u0178': 'Yuml',
        '\uD835\uDCCF': 'zscr',
        '\uD835\uDD37': 'zfr',
        '\uD835\uDD6B': 'zopf',
        '\u2128': 'Zfr',
        '\u2124': 'Zopf',
        '\uD835\uDCB5': 'Zscr',
        '\u017A': 'zacute',
        '\u0179': 'Zacute',
        '\u017E': 'zcaron',
        '\u017D': 'Zcaron',
        '\u017C': 'zdot',
        '\u017B': 'Zdot',
        '\u01B5': 'imped',
        '\xFE': 'thorn',
        '\xDE': 'THORN',
        '\u0149': 'napos',
        '\u03B1': 'alpha',
        '\u0391': 'Alpha',
        '\u03B2': 'beta',
        '\u0392': 'Beta',
        '\u03B3': 'gamma',
        '\u0393': 'Gamma',
        '\u03B4': 'delta',
        '\u0394': 'Delta',
        '\u03B5': 'epsi',
        '\u03F5': 'epsiv',
        '\u0395': 'Epsilon',
        '\u03DD': 'gammad',
        '\u03DC': 'Gammad',
        '\u03B6': 'zeta',
        '\u0396': 'Zeta',
        '\u03B7': 'eta',
        '\u0397': 'Eta',
        '\u03B8': 'theta',
        '\u03D1': 'thetav',
        '\u0398': 'Theta',
        '\u03B9': 'iota',
        '\u0399': 'Iota',
        '\u03BA': 'kappa',
        '\u03F0': 'kappav',
        '\u039A': 'Kappa',
        '\u03BB': 'lambda',
        '\u039B': 'Lambda',
        '\u03BC': 'mu',
        '\xB5': 'micro',
        '\u039C': 'Mu',
        '\u03BD': 'nu',
        '\u039D': 'Nu',
        '\u03BE': 'xi',
        '\u039E': 'Xi',
        '\u03BF': 'omicron',
        '\u039F': 'Omicron',
        '\u03C0': 'pi',
        '\u03D6': 'piv',
        '\u03A0': 'Pi',
        '\u03C1': 'rho',
        '\u03F1': 'rhov',
        '\u03A1': 'Rho',
        '\u03C3': 'sigma',
        '\u03A3': 'Sigma',
        '\u03C2': 'sigmaf',
        '\u03C4': 'tau',
        '\u03A4': 'Tau',
        '\u03C5': 'upsi',
        '\u03A5': 'Upsilon',
        '\u03D2': 'Upsi',
        '\u03C6': 'phi',
        '\u03D5': 'phiv',
        '\u03A6': 'Phi',
        '\u03C7': 'chi',
        '\u03A7': 'Chi',
        '\u03C8': 'psi',
        '\u03A8': 'Psi',
        '\u03C9': 'omega',
        '\u03A9': 'ohm',
        '\u0430': 'acy',
        '\u0410': 'Acy',
        '\u0431': 'bcy',
        '\u0411': 'Bcy',
        '\u0432': 'vcy',
        '\u0412': 'Vcy',
        '\u0433': 'gcy',
        '\u0413': 'Gcy',
        '\u0453': 'gjcy',
        '\u0403': 'GJcy',
        '\u0434': 'dcy',
        '\u0414': 'Dcy',
        '\u0452': 'djcy',
        '\u0402': 'DJcy',
        '\u0435': 'iecy',
        '\u0415': 'IEcy',
        '\u0451': 'iocy',
        '\u0401': 'IOcy',
        '\u0454': 'jukcy',
        '\u0404': 'Jukcy',
        '\u0436': 'zhcy',
        '\u0416': 'ZHcy',
        '\u0437': 'zcy',
        '\u0417': 'Zcy',
        '\u0455': 'dscy',
        '\u0405': 'DScy',
        '\u0438': 'icy',
        '\u0418': 'Icy',
        '\u0456': 'iukcy',
        '\u0406': 'Iukcy',
        '\u0457': 'yicy',
        '\u0407': 'YIcy',
        '\u0439': 'jcy',
        '\u0419': 'Jcy',
        '\u0458': 'jsercy',
        '\u0408': 'Jsercy',
        '\u043A': 'kcy',
        '\u041A': 'Kcy',
        '\u045C': 'kjcy',
        '\u040C': 'KJcy',
        '\u043B': 'lcy',
        '\u041B': 'Lcy',
        '\u0459': 'ljcy',
        '\u0409': 'LJcy',
        '\u043C': 'mcy',
        '\u041C': 'Mcy',
        '\u043D': 'ncy',
        '\u041D': 'Ncy',
        '\u045A': 'njcy',
        '\u040A': 'NJcy',
        '\u043E': 'ocy',
        '\u041E': 'Ocy',
        '\u043F': 'pcy',
        '\u041F': 'Pcy',
        '\u0440': 'rcy',
        '\u0420': 'Rcy',
        '\u0441': 'scy',
        '\u0421': 'Scy',
        '\u0442': 'tcy',
        '\u0422': 'Tcy',
        '\u045B': 'tshcy',
        '\u040B': 'TSHcy',
        '\u0443': 'ucy',
        '\u0423': 'Ucy',
        '\u045E': 'ubrcy',
        '\u040E': 'Ubrcy',
        '\u0444': 'fcy',
        '\u0424': 'Fcy',
        '\u0445': 'khcy',
        '\u0425': 'KHcy',
        '\u0446': 'tscy',
        '\u0426': 'TScy',
        '\u0447': 'chcy',
        '\u0427': 'CHcy',
        '\u045F': 'dzcy',
        '\u040F': 'DZcy',
        '\u0448': 'shcy',
        '\u0428': 'SHcy',
        '\u0449': 'shchcy',
        '\u0429': 'SHCHcy',
        '\u044A': 'hardcy',
        '\u042A': 'HARDcy',
        '\u044B': 'ycy',
        '\u042B': 'Ycy',
        '\u044C': 'softcy',
        '\u042C': 'SOFTcy',
        '\u044D': 'ecy',
        '\u042D': 'Ecy',
        '\u044E': 'yucy',
        '\u042E': 'YUcy',
        '\u044F': 'yacy',
        '\u042F': 'YAcy',
        '\u2135': 'aleph',
        '\u2136': 'beth',
        '\u2137': 'gimel',
        '\u2138': 'daleth'
    },

    regexEscape: /["&'<>`]/g,
    escapeMap: {
        '"': '&quot;',
        //'&': '&amp;',
        '\'': '&#x27;',
        //'<': '&lt;',
        // See https://mathiasbynens.be/notes/ambiguous-ampersands: in HTML, the
        // following is not strictly necessary unless it’s part of a tag or an
        // unquoted attribute value. We’re only escaping it to support those
        // situations, and for XML support.
        //'>': '&gt;',
        // In Internet Explorer ≤ 8, the backtick character can be used
        // to break out of (un)quoted attribute values or HTML comments.
        // See http://html5sec.org/#102, http://html5sec.org/#108, and
        // http://html5sec.org/#133.
        '`': '&#x60;'
    },

    regexInvalidEntity: /&#(?:[xX][^a-fA-F0-9]|[^0-9xX])/,
    regexInvalidRawCodePoint: /[\0-\x08\x0B\x0E-\x1F\x7F-\x9F\uFDD0-\uFDEF\uFFFE\uFFFF]|[\uD83F\uD87F\uD8BF\uD8FF\uD93F\uD97F\uD9BF\uD9FF\uDA3F\uDA7F\uDABF\uDAFF\uDB3F\uDB7F\uDBBF\uDBFF][\uDFFE\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/,
    decodeMapLegacy: {
        'aacute': '\xE1',
        'Aacute': '\xC1',
        'acirc': '\xE2',
        'Acirc': '\xC2',
        'acute': '\xB4',
        'aelig': '\xE6',
        'AElig': '\xC6',
        'agrave': '\xE0',
        'Agrave': '\xC0',
        'amp': '&',
        'AMP': '&',
        'aring': '\xE5',
        'Aring': '\xC5',
        'atilde': '\xE3',
        'Atilde': '\xC3',
        'auml': '\xE4',
        'Auml': '\xC4',
        'brvbar': '\xA6',
        'ccedil': '\xE7',
        'Ccedil': '\xC7',
        'cedil': '\xB8',
        'cent': '\xA2',
        'copy': '\xA9',
        'COPY': '\xA9',
        'curren': '\xA4',
        'deg': '\xB0',
        'divide': '\xF7',
        'eacute': '\xE9',
        'Eacute': '\xC9',
        'ecirc': '\xEA',
        'Ecirc': '\xCA',
        'egrave': '\xE8',
        'Egrave': '\xC8',
        'eth': '\xF0',
        'ETH': '\xD0',
        'euml': '\xEB',
        'Euml': '\xCB',
        'frac12': '\xBD',
        'frac14': '\xBC',
        'frac34': '\xBE',
        'gt': '>',
        'GT': '>',
        'iacute': '\xED',
        'Iacute': '\xCD',
        'icirc': '\xEE',
        'Icirc': '\xCE',
        'iexcl': '\xA1',
        'igrave': '\xEC',
        'Igrave': '\xCC',
        'iquest': '\xBF',
        'iuml': '\xEF',
        'Iuml': '\xCF',
        'laquo': '\xAB',
        'lt': '<',
        'LT': '<',
        'macr': '\xAF',
        'micro': '\xB5',
        'middot': '\xB7',
        'nbsp': '\xA0',
        'not': '\xAC',
        'ntilde': '\xF1',
        'Ntilde': '\xD1',
        'oacute': '\xF3',
        'Oacute': '\xD3',
        'ocirc': '\xF4',
        'Ocirc': '\xD4',
        'ograve': '\xF2',
        'Ograve': '\xD2',
        'ordf': '\xAA',
        'ordm': '\xBA',
        'oslash': '\xF8',
        'Oslash': '\xD8',
        'otilde': '\xF5',
        'Otilde': '\xD5',
        'ouml': '\xF6',
        'Ouml': '\xD6',
        'para': '\xB6',
        'plusmn': '\xB1',
        'pound': '\xA3',
        'quot': '"',
        'QUOT': '"',
        'raquo': '\xBB',
        'reg': '\xAE',
        'REG': '\xAE',
        'sect': '\xA7',
        'shy': '\xAD',
        'sup1': '\xB9',
        'sup2': '\xB2',
        'sup3': '\xB3',
        'szlig': '\xDF',
        'thorn': '\xFE',
        'THORN': '\xDE',
        'times': '\xD7',
        'uacute': '\xFA',
        'Uacute': '\xDA',
        'ucirc': '\xFB',
        'Ucirc': '\xDB',
        'ugrave': '\xF9',
        'Ugrave': '\xD9',
        'uml': '\xA8',
        'uuml': '\xFC',
        'Uuml': '\xDC',
        'yacute': '\xFD',
        'Yacute': '\xDD',
        'yen': '\xA5',
        'yuml': '\xFF'
    },
    decodeMapNumeric: {
        '0': '\uFFFD',
        '128': '\u20AC',
        '130': '\u201A',
        '131': '\u0192',
        '132': '\u201E',
        '133': '\u2026',
        '134': '\u2020',
        '135': '\u2021',
        '136': '\u02C6',
        '137': '\u2030',
        '138': '\u0160',
        '139': '\u2039',
        '140': '\u0152',
        '142': '\u017D',
        '145': '\u2018',
        '146': '\u2019',
        '147': '\u201C',
        '148': '\u201D',
        '149': '\u2022',
        '150': '\u2013',
        '151': '\u2014',
        '152': '\u02DC',
        '153': '\u2122',
        '154': '\u0161',
        '155': '\u203A',
        '156': '\u0153',
        '158': '\u017E',
        '159': '\u0178'
    },
    invalidReferenceCodePoints: [1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 64976, 64977, 64978, 64979, 64980, 64981, 64982, 64983, 64984, 64985, 64986, 64987, 64988, 64989, 64990, 64991, 64992, 64993, 64994, 64995, 64996, 64997, 64998, 64999, 65000, 65001, 65002, 65003, 65004, 65005, 65006, 65007, 65534, 65535, 131070, 131071, 196606, 196607, 262142, 262143, 327678, 327679, 393214, 393215, 458750, 458751, 524286, 524287, 589822, 589823, 655358, 655359, 720894, 720895, 786430, 786431, 851966, 851967, 917502, 917503, 983038, 983039, 1048574, 1048575, 1114110, 1114111],

    /*--------------------------------------------------------------------------*/

    stringFromCharCode: String.fromCharCode,

    object: {},
    hasOwnProperty: function(obj, prop) {
        return obj.hasOwnProperty(prop);
    },
    has: function(object, propertyName) {
        return this.hasOwnProperty.call(object, propertyName);
    },

    contains: function(array, value) {
        var index = -1;
        var length = array.length;
        while (++index < length) {
            if (array[index] == value) {
                return true;
            }
        }
        return false;
    },

    merge: function(options, defaults) {
        if (!options) {
            return defaults;
        }
        var result = {};
        var key;
        for (key in defaults) {
            // A `hasOwnProperty` check is not needed here, since only recognized
            // option names are used anyway. Any others are ignored.
            result[key] = this.has(options, key) ? options[key] : defaults[key];
        }
        return result;
    },

    // Modified version of `ucs2encode`; see https://mths.be/punycode.
    codePointToSymbol: function(codePoint, strict) {
        var output = '';
        if ((codePoint >= 0xD800 && codePoint <= 0xDFFF) || codePoint > 0x10FFFF) {
            // See issue #4:
            // “Otherwise, if the number is in the range 0xD800 to 0xDFFF or is
            // greater than 0x10FFFF, then this is a parse error. Return a U+FFFD
            // REPLACEMENT CHARACTER.”
            if (strict) {
                this.parseError('character reference outside the permissible Unicode range');
            }
            return '\uFFFD';
        }
        if (this.has(this.decodeMapNumeric, codePoint)) {
            if (strict) {
                this.parseError('disallowed character reference');
            }
            return this.decodeMapNumeric[codePoint];
        }
        if (strict && this.contains(this.invalidReferenceCodePoints, codePoint)) {
            this.parseError('disallowed character reference');
        }
        if (codePoint > 0xFFFF) {
            codePoint -= 0x10000;
            output += this.stringFromCharCode(codePoint >>> 10 & 0x3FF | 0xD800);
            codePoint = 0xDC00 | codePoint & 0x3FF;
        }
        output += this.stringFromCharCode(codePoint);
        return output;
    },

    hexEscape: function(codePoint) {
        return '&#x' + codePoint.toString(16).toUpperCase() + ';';
    },

    decEscape: function(codePoint) {
        return '&#' + codePoint + ';';
    },

    parseError: function(message) {
        throw Error('Parse error: ' + message);
    },

    /*--------------------------------------------------------------------------*/

    encode: function(string, options) {
        var self = this;
        var defaultOptions = {
            'allowUnsafeSymbols': false,
            'encodeEverything': false,
            'strict': false,
            'useNamedReferences': false,
            'decimal': false
        };
        options = this.merge(options, defaultOptions);
        var strict = options.strict;
        if (strict && this.regexInvalidRawCodePoint.test(string)) {
            this.parseError('forbidden code point');
        }
        var encodeEverything = options.encodeEverything;
        var useNamedReferences = options.useNamedReferences;
        var allowUnsafeSymbols = options.allowUnsafeSymbols;
        var escapeCodePoint = options.decimal ? this.decEscape : this.hexEscape;

        var escapeBmpSymbol = function(symbol) {
            return escapeCodePoint(symbol.charCodeAt(0));
        };

        if (encodeEverything) {
            // Encode ASCII symbols.
            string = string.replace(this.regexAsciiWhitelist, function(symbol) {
                // Use named references if requested & possible.
                if (useNamedReferences && self.has(self.encodeMap, symbol)) {
                    return '&' + self.encodeMap[symbol] + ';';
                }
                return escapeBmpSymbol(symbol);
            });
            // Shorten a few escapes that represent two symbols, of which at least one
            // is within the ASCII range.
            if (useNamedReferences) {
                string = string
                    .replace(/&gt;\u20D2/g, '&nvgt;')
                    .replace(/&lt;\u20D2/g, '&nvlt;')
                    .replace(/&#x66;&#x6A;/g, '&fjlig;');
            }
            // Encode non-ASCII symbols.
            if (useNamedReferences) {
                // Encode non-ASCII symbols that can be replaced with a named reference.
                string = string.replace(this.regexEncodeNonAscii, function(string) {
                    // Note: there is no need to check `has(encodeMap, string)` here.
                    return '&' + self.encodeMap[string] + ';';
                });
            }
            // Note: any remaining non-ASCII symbols are handled outside of the `if`.
        } else if (useNamedReferences) {
            // Apply named character references.
            // Encode `<>"'&` using named character references.
            if (!allowUnsafeSymbols) {
                string = string.replace(this.regexEscape, function(string) {
                    return '&' + self.encodeMap[string] + ';'; // no need to check `has()` here
                });
            }
            // Shorten escapes that represent two symbols, of which at least one is
            // `<>"'&`.
            s//tring = string.replace(/&gt;\u20D2/g, '&nvgt;').replace(/&lt;\u20D2/g, '&nvlt;');
            // Encode non-ASCII symbols that can be replaced with a named reference.
            string = string.replace(this.regexEncodeNonAscii, function(string) {
                // Note: there is no need to check `has(encodeMap, string)` here.
                return '&' + self.encodeMap[string] + ';';
            });
        } else if (!allowUnsafeSymbols) {
            // Encode `<>"'&` using hexadecimal escapes, now that they’re not handled
            // using named character references.
            string = string.replace(this.regexEscape, escapeBmpSymbol);
        }
        return string.replace(this.regexAstralSymbols, function($0) {
            // https://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
            var high = $0.charCodeAt(0);
            var low = $0.charCodeAt(1);
            var codePoint = (high - 0xD800) * 0x400 + low - 0xDC00 + 0x10000;
            return escapeCodePoint(codePoint);
        }).replace(this.regexBmpWhitelist, escapeBmpSymbol);
        // Encode any remaining BMP symbols that are not printable ASCII symbols
        // using a hexadecimal escape.

    },

    decode: function(html, options) {
        var self = this;
        var defaultOptions = {
            'isAttributeValue': false,
            'strict': false
        };
        options = this.merge(options, defaultOptions);
        var strict = options.strict;
        if (strict && this.regexInvalidEntity.test(html)) {
            this.parseError('malformed character reference');
        }
        return html.replace(this.regexDecode, function($0, $1, $2, $3, $4, $5, $6, $7, $8) {
            var codePoint;
            var semicolon;
            var decDigits;
            var hexDigits;
            var reference;
            var next;

            if ($1) {
                reference = $1;
                // Note: there is no need to check `has(decodeMap, reference)`.
                return self.decodeMap[reference];
            }

            if ($2) {
                // Decode named character references without trailing `;`, e.g. `&amp`.
                // This is only a parse error if it gets converted to `&`, or if it is
                // followed by `=` in an attribute context.
                reference = $2;
                next = $3;
                if (next && options.isAttributeValue) {
                    if (strict && next == '=') {
                        self.parseError('`&` did not start a character reference');
                    }
                    return $0;
                } else {
                    if (strict) {
                        self.parseError(
                            'named character reference was not terminated by a semicolon'
                        );
                    }
                    // Note: there is no need to check `has(decodeMapLegacy, reference)`.
                    return self.decodeMapLegacy[reference] + (next || '');
                }
            }

            if ($4) {
                // Decode decimal escapes, e.g. `&#119558;`.
                decDigits = $4;
                semicolon = $5;
                if (strict && !semicolon) {
                    self.parseError('character reference was not terminated by a semicolon');
                }
                codePoint = parseInt(decDigits, 10);
                return self.codePointToSymbol(codePoint, strict);
            }

            if ($6) {
                // Decode hexadecimal escapes, e.g. `&#x1D306;`.
                hexDigits = $6;
                semicolon = $7;
                if (strict && !semicolon) {
                    self.parseError('character reference was not terminated by a semicolon');
                }
                codePoint = parseInt(hexDigits, 16);
                return self.codePointToSymbol(codePoint, strict);
            }

            // If we’re still here, `if ($7)` is implied; it’s an ambiguous
            // ampersand for sure. https://mths.be/notes/ambiguous-ampersands
            if (strict) {
                self.parseError(
                    'named character reference was not terminated by a semicolon'
                );
            }
            return $0;
        });
    },

    escape: function(string) {
        var self = this;
        return string.replace(this.regexEscape, function($0) {
            // Note: there is no need to check `has(escapeMap, $0)` here.
            return self.escapeMap[$0];
        });
    },
    version: '1.2.0'
    /*--------------------------------------------------------------------------*/

};
var he = htmlEntities;

// ----------------------------- END NATIVE ---------------------------
GlobalState = State();
//doc = Document("{\\rtf1\\ansi\\ansicpg1252\\deff0\\nouicompat {\\fonttbl {\\f0\\fnil\\fcharset0 Arial;} {\\f1\\fnil\\fcharset0 Arial Black;} {\\f2\\fnil\\fcharset0 Courier New;} {\\f3\\fnil\\fcharset0 Georgia;} {\\f4\\fnil\\fcharset0 Tahoma;} {\\f5\\fnil\\fcharset0 Times New Roman;} {\\f6\\fnil\\fcharset0 Verdana;} } {\\colortbl ;} {\\pard {\\fs10} {\\b\\fs10 Negrito} {\\fs10 \\~} {\\i\\fs10 Italico} {\\fs10 \\~} {\\ul\\fs10 Sublinhado} \\line\\sb70\\par} {\\pard\\fs7 {\\fs10 Esquerda} \\sb70\\par} {\\pard\\fs7\\qc {\\fs10 Centro}\\sb70\\par}{\\pard\\fs7\\qr{\\fs10 Direita\\line}\\sb70\\par} }");
//var strHtml = HtmlFormatter().Format(doc);
function convertRtfToHtml(stringRtf) {
    GlobalState = State();
    // stringRtf = "{\\rtf1\\ansi\\ansicpg1252\\deff0\\nouicompat {\\fonttbl {\\f0\\fnil\\fcharset0 Arial;} {\\f1\\fnil\\fcharset0 Arial Black;} {\\f2\\fnil\\fcharset0 Courier New;} {\\f3\\fnil\\fcharset0 Georgia;} {\\f4\\fnil\\fcharset0 Tahoma;} {\\f5\\fnil\\fcharset0 Times New Roman;} {\\f6\\fnil\\fcharset0 Verdana;} } {\\colortbl ;} {\\pard {\\fs10} {\\b\\fs10 Negrito} {\\i\\fs10 Italico}{\\ul\\fs10 Sublinhado}\\line\\sb70\\par}{\\pard\\fs7{\\fs10 Esquerda}\\sb70\\par}{\\pard\\fs7\\qc{\\fs10 Centro}\\sb70\\par}{\\pard\\fs7\\qr{\\fs10 Direita\\line}\\sb70\\par}}";
    var doc = Document(stringRtf.split(chr(0)).join(""));
    return HtmlFormatter().Format(doc);
}

function isRtf(str) {
    return str.substr(2, 3) == "rtf";
}

//Utilizando "-" para substituir as bolinhas quando cria uma lista

function strip_rtf_markup(teste) {
	// htmlString = htmlString.replace(/\\par[d]?/g, "");
	//htmlString = htmlString.replace(/\{\*?\\[^{}]+}|[{}]|\\\n?[A-Za-z]+\n?(?:-?\d+)?[ ]?/g, "");
    teste = teste.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, ''); //substitui o trim que nao estava reconhecendo pelo usuario
	find = ["<p>", "</p>", "ￂﾰ", "ￂﾪ", "ￃﾧ", "ￃﾇ", "ￂﾺ", "ￂﾣ", "ￂﾢ","ￂﾨ", "ￂﾬ", "ￂﾧ", "ￃﾡ", "ￃﾩ", "ￃﾭ", "ￃﾳ", "ￃﾺ", "ￃﾁ", "ￃﾉ", "ￃﾍ", "ￃﾓ", "ￃﾚ", "ￂﾴ", "\\\\\'20","\\\\\'21","\\\\\'22","\\\\\'23","\\\\\'24","\\\\\'25","\\\\\'26","\\\\\'27","\\\\\'28","\\\\\'29","\\\\\'2a","\\\\\'2b","\\\\\'2c","\\\\\'2d","\\\\\'2e","\\\\\'2f","\\\\\'30","\\\\\'31","\\\\\'32","\\\\\'33","\\\\\'34","\\\\\'35","\\\\\'36","\\\\\'37","\\\\\'38","\\\\\'39","\\\\\'3a","\\\\\'3b","\\\\\'3c","\\\\\'3d","\\\\\'3e","\\\\\'3f","\\\\\'40","\\\\\'41","\\\\\'42","\\\\\'43","\\\\\'44","\\\\\'45","\\\\\'46","\\\\\'47","\\\\\'48","\\\\\'49","\\\\\'4a","\\\\\'4b","\\\\\'4c","\\\\\'4d","\\\\\'4e","\\\\\'4f","\\\\\'50","\\\\\'51","\\\\\'52","\\\\\'53","\\\\\'54","\\\\\'55","\\\\\'56","\\\\\'57","\\\\\'58","\\\\\'59","\\\\\'5a","\\\\\'5b","\\\\\'5c","\\\\\'5d","\\\\\'5e","\\\\\'5f","\\\\\'60","\\\\\'61","\\\\\'62","\\\\\'63","\\\\\'64","\\\\\'65","\\\\\'66","\\\\\'67","\\\\\'68","\\\\\'69","\\\\\'6a","\\\\\'6b","\\\\\'6c","\\\\\'6d","\\\\\'6e","\\\\\'6f","\\\\\'70","\\\\\'71","\\\\\'72","\\\\\'73","\\\\\'74","\\\\\'75","\\\\\'76","\\\\\'77","\\\\\'78","\\\\\'79","\\\\\'7a","\\\\\'7b","\\\\\'7c","\\\\\'7d","\\\\\'7e","\\\\\'7f","\\\\\'80","\\\\\'81","\\\\\'82","\\\\\'83","\\\\\'84","\\\\\'85","\\\\\'86","\\\\\'87","\\\\\'88","\\\\\'89","\\\\\'8a","\\\\\'8b","\\\\\'8c","\\\\\'8d","\\\\\'8e","\\\\\'8f","\\\\\'90","\\\\\'91","\\\\\'92","\\\\\'93","\\\\\'94","\\\\\'95","\\\\\'96","\\\\\'97","\\\\\'98","\\\\\'99","\\\\\'9a","\\\\\'9b","\\\\\'9c","\\\\\'9d","\\\\\'9e","\\\\\'9f","\\\\\'~","\\\\\'a1","\\\\\'a2","\\\\\'a3","\\\\\'a4","\\\\\'a5","\\\\\'a6","\\\\\'a7","\\\\\'a8","\\\\\'a9","\\\\\'aa","\\\\\'ab","\\\\\'ac","\\\\\'-","\\\\\'ae","\\\\\'af","\\\\\'b0","\\\\\'b1","\\\\\'b2","\\\\\'b3","\\\\\'b4","\\\\\'b5","\\\\\'b6","\\\\\'b7","\\\\\'b8","\\\\\'b9","\\\\\'ba","\\\\\'bb","\\\\\'bc","\\\\\'bd","\\\\\'be","\\\\\'bf","\\\\\'c0","\\\\\'c1","\\\\\'c2","\\\\\'c3","\\\\\'c4","\\\\\'c5","\\\\\'c6","\\\\\'c7","\\\\\'c8","\\\\\'c9","\\\\\'ca","\\\\\'cb","\\\\\'cc","\\\\\'cd","\\\\\'ce","\\\\\'cf","\\\\\'d0","\\\\\'d1","\\\\\'d2","\\\\\'d3","\\\\\'d4","\\\\\'d5","\\\\\'d6","\\\\\'d7","\\\\\'d8","\\\\\'d9","\\\\\'da","\\\\\'db","\\\\\'dc","\\\\\'dd","\\\\\'de","\\\\\'df","\\\\\'e0","\\\\\'e1","\\\\\'e2","\\\\\'e3","\\\\\'e4","\\\\\'e5","\\\\\'e6","\\\\\'e7","\\\\\'e8","\\\\\'e9","\\\\\'ea","\\\\\'eb","\\\\\'ec","\\\\\'ed","\\\\\'ee","\\\\\'ef","\\\\\'f0","\\\\\'f1","\\\\\'f2","\\\\\'f3","\\\\\'f4","\\\\\'f5","\\\\\'f6","\\\\\'f7","\\\\\'f8","\\\\\'f9","\\\\\'fa","\\\\\'fb","\\\\\'fc","\\\\\'fd","\\\\\'fe","\\\\\'ff", "\\\\\\*"];
	replace = [" ", " ", "°", "ª", "ç", "Ç", "º", "£", "¢", "¨", "¬", "§", "á", "é", "í", "ó", "ú", "Á", "É", "Í", "Ó", "Ú", "´", " ","!",'"',"#","$","%","&","'","(",")","*","+",",","-",".","/","0","1","2","3","4","5","6","7","8","9",":",";","<","=",">","?","@","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","[","\\","]","^","_","`","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z","{","|","}","~"," ",","," ","͵","ƒ",",,,","...","†","‡","∘","‰","Š","‹","Œ"," ","Ž"," "," ","‘","’","“","”","•","–","—","~","™","š","›","œ"," ","ž","Ÿ"," ","¡","¢","£","¤","¥","¦","§","¨","©","ª","«","¬","(-)","®","¯","°","±","²","³","´","µ","¶","·","¸","¹","º","»","¼","½","¾","¿","À","Á","Â","Ã","Ä","Å","Æ","Ç","È","É","Ê","Ë","Ì","Í","Î","Ï","Ð","Ñ","Ò","Ó","Ô","Õ","Ö","×","Ø","Ù","Ú","Û","Ü","Ý","Þ","ß","à","á","â","ã","ä","å","æ","ç","è","é","ê","ë","ì","í","î","ï","ð","ñ","ò","ó","ô","õ","ö","÷","ø","ù","ú","û","ü","ý","þ","ÿ", "*"];
	for(i = 0; i < find.length; i++){
		teste = teste.replace(new RegExp(find[i], 'g'), replace[i]);
	}
	return teste;
}

function rtfToHtml(byteArr) {
    //return strHtml;//+"n/>";
    var teste = "";
    for (var i = 0; i < byteArr.length; i++) {
        teste += String.fromCharCode(byteArr[i]);
    }
	
	 var pos = teste.indexOf("rtf");
	 teste = strip_rtf_markup(teste);
	
    if (isRtf(teste) && pos != -1) {
        return convertRtfToHtml(teste);
    } else teste = teste.replace(/\n/g, '\n<br>');
    
	return teste;
}

function htmlToHtml(byteArr) {
    texto = strip_rtf_markup(byteArr);
	return texto;
}