import {
  Http, Config, Loader, I18n, IDictionary, dayjs as moment, Singleton,
} from '@zeedhi/core';

import { PmeCurrencyMask, PmeMask } from '@/config';
import 'dayjs/locale/pt';
import 'dayjs/locale/en';
import 'dayjs/locale/es';

@Singleton
export class LoginController {
  get logo() {
    return Config.env.product.logoUrl;
  }

  get backgroundImgUrl() {
    const url = Config.env.loginBackgroundImgUrl;
    return `url(${url})`;
  }

  get endPoint() {
    return Config.env.loginEndPoint;
  }
  
  public utilities = Loader.getInstance('Utilities');

  public appController = Loader.getInstance('AppController');

  private get language() {
    return localStorage.getItem('LOGIN_LANGUAGE');
  }

  get backgroundStyle(): string {
    return `url(${Config.env.loginBackground})`;
  }
  
  public setLocalStorageUser(userData: IDictionary, libInfo: IDictionary) {
    if (userData) {
      this.appController.setUniDefault();
      this.utilities.initalizeDtProcess();
      Http.post('/userInfo', { libInfo }).then((response) => {
        const userInfo = response.data?.dataset?.userInfo;
        if (userInfo) {
          localStorage.setItem('CDFILIALLOGADA', userInfo.CDFILIALSELECIONADA);
          localStorage.setItem('NMFILIALLOGADA', userInfo.NMFILIAL);
          
          localStorage.setItem('CDPICTPROD', userInfo.CDPICTPROD);
          localStorage.setItem('CDPICTPRAT', userInfo.CDPICTPRAT);
          // Adicionado tratamento para impedir que o mesmo não informe o valor corretamente
          (Config.masks.productMask as PmeMask).pattern = userInfo.CDPICTPROD
            ? userInfo.CDPICTPROD.replace(/[0-9a-zA-Z!@#$%^&*()_+]/g, 'N')
            : 'N.NN.NN.NNN.NN';
          
          (Config.masks.productMask as PmeMask).nivelMax = userInfo.CDPICTPROD
            ? userInfo.CDPICTPROD.split('.').length
            : 5;
          
          this.utilities.cdCentralLogada = userInfo.CDFILIALSELECIONADA;
          this.utilities.nmCentralLogada = userInfo.NMFILIAL;
        }
      });
  
      this.setLanguage();
      
      localStorage.setItem('DATEFORMAT', this.utilities.getFormatLocale());
      Config.displayFormat = this.utilities.getFormatLocale();
      
      const lang = this.utilities.getLanguage();
      const currency: string = this.appController.metaCurrency[lang];
      const locale: string = this.appController.metaLanguages[lang];
      const separador = localStorage.getItem('LOGIN_SEPARATOR') || '.,';
      
      (Config.masks.currencyMask as PmeCurrencyMask).currencySymbol = this.utilities.getCurrencySymbol(locale, currency);
      (Config.masks.currencyMask as PmeCurrencyMask).decimalCharacter = separador.substring(2, 1) || ',';
      (Config.masks.currencyMask as PmeCurrencyMask).digitGroupSeparator = separador.substring(0, 1) || '.';
      
      (Config.masks.numberMask as PmeCurrencyMask).decimalCharacter = separador.substring(2, 1) || ',';
      (Config.masks.numberMask as PmeCurrencyMask).digitGroupSeparator = separador.substring(0, 1) || '.';
      
      localStorage.setItem('CURRENCY', (Config.masks.currencyMask as PmeCurrencyMask).currencySymbol);
    }
  }
  
  private setLanguage() {
    const lang = this.utilities.getLanguage();
    I18n.changeLanguage(this.appController.metaLanguages[lang]);
    moment.locale(lang.substring(0, 2));
  }
}
