import {
  Loader, Metadata, IDictionary, IEventParam, Http, Config,
} from '@zeedhi/core';

import {
  Form, Select, Grid, IDate, IGridEvent, GridEditable, TextInput, ModalService, Modal,
} from '@zeedhi/common';

import { PmeMask } from '@/config';

export class Pme12300RestricaoProdutos {
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
    
  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public modalVigencia!: Modal;
  
  private selectedRestProd = '';
  
  private descricao = '';
  
  private pratoIsVisible = false;
  
  private maxFreqGrPr = '';
  
  private selectedAgrup = '';
  
  private newValidVigencia = true;
  
  public newPorProduto({ component }: IEventParam<Grid>) {
    const form = Metadata.getInstance<Form>('formRestProd');
    if (form.validate()) this.gridEditController.addRow({ component });
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const servico = Metadata.getInstance<Select>('SERVICO');
    const formGrupoPrato = Metadata.getInstance<Form>('formRestProd');
    
    if (component.name === 'TIPOCARD') {
      servico.setValue('');
      this.selectedRestProd = '';
      formGrupoPrato.value.describeCard = null;
      formGrupoPrato.value.finalDate = null;
      formGrupoPrato.value.initialDate = null;
      this.setFilterGrids();
    }
  }
  
  public clearGrupo({ component }: IEventParam<Select>) {
    this.selectedAgrup = component.value;
    
    const grupo = Metadata.getInstance<Select>('NMGRUPOPRATOPAE_editable_');
    grupo.setValue('');
    grupo.datasource.filter = this.setFilterAgrup();
    grupo.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const cdtipoCard:Select = Metadata.getInstance('TIPOCARD-RestProd');
    let valueCard;
    
    if (cdtipoCard.returnObject === true && cdtipoCard.value !== null) valueCard = cdtipoCard.value.CDTIPOCARD;
    else valueCard = cdtipoCard.value;
    
    const conditions = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? valueCard : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public resetDate({ component }: IEventParam<IDictionary>) {
    component.value = null;
    const fim: IDate = Metadata.getInstance<IDate>('finalDate-RestProd');
    fim.value = null;
    this.selectedRestProd = '';
  }
  
  public changeVigencia() {
    const tipoCard: IDictionary = Metadata.getInstance('TIPOCARD-RestProd');
    const servico: IDictionary = Metadata.getInstance('SERVICO-RestProd');
    const ini: IDate = Metadata.getInstance<IDate>('initialDate-RestProd');
    const fim: IDate = Metadata.getInstance<IDate>('finalDate-RestProd');
    if (ini.value !== null || (ini.value !== null && fim.value !== null && this.utilities.dateIsValid(ini, fim))) {
      if (tipoCard.value && servico.value) {
        const filter: IDictionary = [];
        this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', tipoCard.value);
        this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
        this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', servico.value);
        
        Http.post('/find/vigenciaRestProd', { requestType: 'FilterData', filter }).then((response) => {
          const result = response.data.data;
          // trata data de vigencia selecionada, nao pode intercalar com outra
          let nrRestProd = '';
          let desc = '';
          let finalDate = fim.value;
          result.forEach((vigencia: IDictionary) => {
            if (vigencia.DTVIGENCIA === ini.value && vigencia.DTVIGENCIAFIM === fim.value) {
              nrRestProd = vigencia.NRRESTPRODPAE;
              desc = vigencia.DESCRICAO;
              finalDate = vigencia.DTVIGENCIAFIM;
            }
          });
          this.selectedRestProd = nrRestProd;
          this.descricao = desc;
          Metadata.getInstance<IDictionary>('description-RestProd').value = desc;
          fim.value = finalDate;
          if (this.selectedRestProd === '' && !this.utilities.validateVigenciaDates(result, ini, fim)) {
            this.utilities.showAlert('alertError', 'pme::dataVigenciaIntercalada', 'error');
            this.newValidVigencia = false;
          } else {
            this.newValidVigencia = true;
          }
        }).finally(() => {
          this.setFilterGrids();
        });
      } else {
        this.utilities.showAlert('alertError', 'pme::tpCardServNaoSelecionado', 'warning');
        this.setFilterGrids();
      }
    } else this.setFilterGrids();
  }
  
  public setVigencia({ row }: IGridEvent) {
    if (row) {
      const ini: IDate = Metadata.getInstance<IDate>('initialDate-RestProd');
      const fim: IDate = Metadata.getInstance<IDate>('finalDate-RestProd');
      ini.value = row.DTVIGENCIA;
      fim.value = row.DTVIGENCIAFIM;
      this.selectedRestProd = row.NRRESTPRODPAE;
      
      const descricao: IDictionary = Metadata.getInstance('description-RestProd');
      descricao.value = row.DESCRICAO;
      
      this.descricao = row.DESCRICAO;
      this.changeVigencia();
      this.modalVigencia.hide();
    }
  }
  
  public setFilterGrids() {
    const filter = [
      {
        name: 'NRRESTPRODPAE',
        operator: '=',
        value: this.selectedRestProd,
      },
    ];
    if (document.getElementsByName('gridRestProd').length > 0) {
      const gridProd: Grid = Metadata.getInstance('gridRestProd');
      gridProd.datasource.filter = filter;
      gridProd.datasource.get();
    }
    if (document.getElementsByName('gridRestProdGrup').length > 0) {
      const gridProdPorGrup: Grid = Metadata.getInstance('gridRestProdGrup');
      gridProdPorGrup.datasource.filter = filter;
      gridProdPorGrup.datasource.get();
    }
  }
  
  private async createModalVigencia() {
    const modalPath = 'pme/restricoesContratuais/cadastros/modals/gridVigenciasRestProd';
    this.modalVigencia = await ModalService.createFromJson(modalPath, true);
  }
  
  public openVigenciaModal() {
    const tipoCard: IDictionary = Metadata.getInstance('TIPOCARD-RestProd');
    const servico: IDictionary = Metadata.getInstance('SERVICO-RestProd');
    if (tipoCard.value && servico.value) {
      if (!this.modalVigencia) {
        this.createModalVigencia().then(() => {
          this.modalVigencia.show();
          this.setFilterVigencia();
        });
      } else {
        this.modalVigencia.show();
        this.setFilterVigencia();
      }
    } else {
      this.utilities.showAlert('alertError', 'pme::tpCardServNaoSelecionado', 'warning');
    }
  }
  
  public closeModalVigencia() {
    this.modalVigencia.hide();
  }
  
  public setFilterVigencia() {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD-RestProd');
    const cdservfili = Metadata.getInstance<Select>('SERVICO-RestProd');
    
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? cdtipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdservfili.value ? cdservfili.value : '',
      },
    ];
    
    if (document.getElementsByName('gridVigenciaRestProd').length > 0) {
      const gridVigenciaProd = Metadata.getInstance<Grid>('gridVigenciaRestProd');
      if (gridVigenciaProd) {
        gridVigenciaProd.datasource.filter = filter;
        gridVigenciaProd.datasource.get();
      }
    }
  }

  public getFilterProd() {
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
  }
  
  public savePorProduto() {
    const gridRestProd: GridEditable = Metadata.getInstance('gridRestProd');
    const form = Metadata.getInstance<Form>('formRestProd');
    const editedRows: IDictionary = gridRestProd.getEditedRows();
    if (this.selectedRestProd === '') {
      if (!this.newValidVigencia) this.utilities.showAlert('alertError', 'pme::dataVigenciaIntercalada', 'error');
      else if (form.validate() && editedRows.length > 0) {
        const ini: IDate = Metadata.getInstance<IDate>('initialDate-RestProd');
        const fim: IDate = Metadata.getInstance<IDate>('finalDate-RestProd');
        const descricao: IDictionary = Metadata.getInstance('description-RestProd');
        
        const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD-RestProd');
        const cdservfili = Metadata.getInstance<Select>('SERVICO-RestProd');
        
        Http.post('/returnCodRestProd', {}).then((response: IDictionary) => {
          const codigo = response.data.dataset.codigo[0].CODIGO;
          
          const vigencia = {
            NRRESTPRODPAE: codigo + 1,
            DTVIGENCIA: ini.value,
            DTVIGENCIAFIM: fim.value,
            DESCRICAO: descricao.value,
            CDFILCENTPME: this.appController.getCdCentral(),
            CDTIPOCARD: cdtipoCard.value,
            CDSERVFILI: cdservfili.value,
            __is_new: true,
          };
          
          this.crudController.saveRows([vigencia], 'vigenciaRestProd', gridRestProd).then((result: IDictionary) => {
            if (result.data.dataset) {
              this.selectedRestProd = vigencia.NRRESTPRODPAE;
              this.saveItRestProd(gridRestProd);
              // this.gridEditController.cancelEditing({ component: gridRestProd });
            } else {
              const error = response.data.error;
              if (error.indexOf('UN_RESTPRODPAE') > 0) {
                this.utilities.showAlert('alertError', 'pme::vigenciaJaExiste', 'error');
              } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        });
      }
    } else {
      this.saveItRestProd(gridRestProd);
    }
  }
  
  private saveItRestProd(grid: GridEditable) {
    const form = Metadata.getInstance<Form>('formRestProd');
    const editedRows: IDictionary = grid.getEditedRows();
    if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        row.NRRESTPRODPAE = this.selectedRestProd;
      });
      
      Http.post('/saveItRestProd', { editedRows }).then(() => {
        this.setFilterGrids();
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
        this.gridEditController.cancelEditing({ component: grid });
      }).catch((error: string) => {
        if (error.indexOf('UN_ITRESTPRODPAE') > 0) {
          this.utilities.showAlert('alertError', 'pme::impossivelAdicionar', 'error');
        } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValue: 'CDPRODUTO',
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/prodAgregAtivo',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public savePorRestProdGrup() {
    const gridRestProdGrup: GridEditable = Metadata.getInstance('gridRestProdGrup');
    const form = Metadata.getInstance<Form>('formRestProd');
    const editedRows: IDictionary = gridRestProdGrup.getEditedRows();
    if (this.selectedRestProd === '') {
      if (!this.newValidVigencia) this.utilities.showAlert('alertError', 'pme::dataVigenciaIntercalada', 'error');
      else if (form.validate() && editedRows.length > 0) {
        const ini: IDate = Metadata.getInstance<IDate>('initialDate-RestProd');
        const fim: IDate = Metadata.getInstance<IDate>('finalDate-RestProd');
        const descricao: IDictionary = Metadata.getInstance('description-RestProd');
        
        const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD-RestProd');
        const cdservfili = Metadata.getInstance<Select>('SERVICO-RestProd');
        
        Http.post('/returnCodRestProd', {}).then((response: IDictionary) => {
          const codigo = response.data.dataset.codigo[0].CODIGO;
          
          const vigencia = {
            NRRESTPRODPAE: codigo + 1,
            DTVIGENCIA: ini.value,
            DTVIGENCIAFIM: fim.value,
            DESCRICAO: descricao.value,
            CDFILCENTPME: this.appController.getCdCentral(),
            CDTIPOCARD: cdtipoCard.value,
            CDSERVFILI: cdservfili.value,
            __is_new: true,
          };
          
          this.crudController.saveRows([vigencia], 'vigenciaRestProd', gridRestProdGrup).then((result: IDictionary) => {
            if (result.data.dataset) {
              this.selectedRestProd = vigencia.NRRESTPRODPAE;
              this.saveItRestProdGrup(gridRestProdGrup);
              // this.gridEditController.cancelEditing({ component: gridRestProdGrup });
            } else {
              const error = response.data.error;
              if (error.indexOf('UN_RESTPRODPAE') > 0) {
                this.utilities.showAlert('alertError', 'pme::vigenciaJaExiste', 'error');
              } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        });
      }
    } else {
      this.saveItRestProdGrup(gridRestProdGrup);
    }
  }
  
  private saveItRestProdGrup(grid: GridEditable) {
    const form = Metadata.getInstance<Form>('formRestProd');
    const editedRows: IDictionary = grid.getEditedRows();
    if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        row.NRRESTPRODPAE = this.selectedRestProd;
      });
      
      Http.post('/saveItRestProdGrup', { editedRows }).then(() => {
        this.setFilterGrids();
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
        this.gridEditController.cancelEditing({ component: grid });
      }).catch((error: string) => {
        if (error.indexOf('UN_ITRESTPRODGRPRPAE') > 0) {
          this.utilities.showAlert('alertError', 'pme::impossivelAdicionar', 'error');
        } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  private setFilterAgrup() {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD-RestProd');
    const cdservfili = Metadata.getInstance<Select>('SERVICO-RestProd');
    
    return [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? cdtipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cdservfili.value ? cdservfili.value : '',
      },
      {
        name: 'NRAGRUPGRPRAT',
        operator: '=',
        value: this.selectedAgrup,
      },
    ];
  }
  
  public changeDescription({ component }: IEventParam<TextInput>) {
    if (this.selectedRestProd !== '' && component.value !== this.descricao) {
      const ini: IDate = Metadata.getInstance<IDate>('initialDate-RestProd');
      const fim: IDate = Metadata.getInstance<IDate>('finalDate-RestProd');
      const descricao: IDictionary = Metadata.getInstance('description-RestProd');
      const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD-RestProd');
      const cdservfili = Metadata.getInstance<Select>('SERVICO-RestProd');
      if (this.selectedRestProd !== '') {
        const vigencia = {
          NRRESTPRODPAE: this.selectedRestProd,
          DTVIGENCIA: ini.value,
          DTVIGENCIAFIM: fim.value,
          DESCRICAO: descricao.value,
          CDFILCENTPME: this.appController.getCdCentral(),
          CDTIPOCARD: cdtipoCard.value,
          CDSERVFILI: cdservfili.value,
          __is_new: false,
        };
        
        this.crudController.saveRows([vigencia], 'vigenciaRestProd').then(() => {
          this.descricao = descricao.value;
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
}
